package no.ingeberg.NewsML;
import java.net.*;
/**
 *ContentItem.java <br>
 * Klassen hndterer innkomne data under "ContentItem" elementet, som er et nyhetsobjekt som identoifiserer innhold 
 * (slik som tekst, bilder, video, lyd etc) ment for presentasjon for mennesker.
 * Klassen innholder og metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */
public class ContentItem {
    private URL contentItemUrl              = null;
    private String comment                  = null;
    private Catalog catalog                 = null;
    private String mediaTypeFN              = null;
    private String mediaTypeScheme          = null;
    private String formatFN                 = null;
    private String formatScheme             = null;
    private String mimeTypeFN               = null;
    private String mimeTypeScheme           = null;
    private String notationFN               = null;
    private String notationScheme           = null;
    private String sizeInBytes              = null;
    private String propertyFN               = null;
    private String propertyValue            = null;
    private ContentItemSub contentItemSub   = null;
    
    
    /**
     *Oppretter en ny instans av ContentItem, i denne konstruktoren er alle under elementene tatt med
     * @param contentItemUrl
     * @param comment           kommentar element. <br><br>
     * @param catalog            Identifiserer standard vokabular for ContetItem. <br>
     *                           Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                            Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param mediaTypeFN       Sammen med mediaTypeScheme utgjr dette elementet mediaType, som angir skjema og innhold som beskriver hvilken 
     *                          mediatype innholdet er av.<br><br>
     * @param mediaTypeScheme   Sammen med mediaTypeFN utgjr dette elementet mediaType, som angir skjema og innhold som beskriver hvilken 
     *                          mediatype innholdet er av.<br><br>
     * @param formatFN          Sammen med formatScheme utgjr dette elementet format, som angir skjema og innhold som beskriver hvilket format 
     *                          den aktuele innholdsposten er av.<br><br>
     * @param formatScheme      Sammen med formatFN utgjr dette elementet format, som angir skjema og innhold som beskriver hvilket format den 
     *                          aktuele innholdsposten er av.<br><br>
     * @param mimeTypeFN        Sammen med mimeTypeScheme utgjr dette elementet mimeType, som angir skjema og innhold som beskriver hvilken 
     *                          mimetype den aktuele innholdsposten er av.<br><br>
     * @param mimeTypeScheme    Sammen med mimeTypeFN utgjr dette elementet mimeType, som angir skjema og innhold som beskriver hvilken mimetype 
     *                          den aktuele innholdsposten er av.<br><br>
     * @param notationFN        Sammen med notationScheme utgjr dette elementet notation, som angir skjema og innhold som beskriver hvilken 
     *                          notasjon den aktuele nyhetsposten er av.<br><br>
     * @param notationScheme    Sammen med notationFN utgjr dette elementet notation, som angir skjema og innhold som beskriver hvilken 
     *                          notasjon den aktuele nyhetsposten er av.<br><br>
     * @param sizeInBytes       Angir strrelsen p fila.<br><br>
     * @param propertyFN        Angir navnet for spesifikke egenskaper ved innholdet, som ikke er dekket av de vrige. <br><br>
     * @param propertyValue     Angir navnet for spesifikke egenskaper ved innholdet, som ikke er dekket av de vrige. <br><br>
     *
     * @param data              Angir selve innholdselementet. -Ingen tegn i dette elementet m kunne forveksles med XML, med mindre de er 
     *                          "pakket inne" i hendholdt til xml-spesifiasjonen.<br><br>
     */
    //  * @param encoding          Angir data komprimeringen p innholdselementet. er ikke tatt med da enkoding og data ikke kan forekomme 
    //                                sammen.<br><br>
    public ContentItem( URL contentItemUrl, String comment, Catalog catalog,
    String mediaTypeFN, String mediaTypeScheme, String formatFN,
    String formatScheme, String mimeTypeFN, String mimeTypeScheme,
    String notationFN, String notationScheme, String sizeInBytes,
    String propertyFN, String propertyValue, String input, boolean isData){
        
        this.contentItemUrl     = contentItemUrl;
        this.comment            = comment;
        this.catalog            = catalog;
        this.mediaTypeFN        = mediaTypeFN;
        this.mediaTypeScheme    = mediaTypeScheme;
        this.formatFN           = formatFN;
        this.formatScheme       = formatScheme;
        this.mimeTypeFN         = mimeTypeFN;
        this.mimeTypeScheme     = mimeTypeScheme;
        this.notationFN         = notationFN;
        this.notationScheme     = notationScheme;
        this.sizeInBytes        = sizeInBytes;
        this.propertyFN         = propertyFN;
        this.propertyValue      = propertyValue;
        this.contentItemSub     = new ContentItemSub(input, isData);
        
    }
    
     /**
     *Oppretter en ny instans av ContentItem, i denne konstruktoren er alle under elementene tatt med   
     * @param comment           kommentar element. <br><br>
     * @param catalog           Identifiserer standard vokabular for ContetItem. <br>
     * @param mediaTypeFN       Sammen med mediaTypeScheme utgjr dette elementet mediaType, som angir skjema og innhold som beskriver hvilken 
      *                         media type innholdet er av.<br><br>
     * @param formatFN          Sammen med formatScheme utgjr dette elementet format, som angir skjema og innhold som beskriver hvilket format 
      *                         den aktuele innholdsposten er av.<br><br>
     * @param mimeTypeFN        Sammen med mimeTypeScheme utgjr dette elementet mimeType, som angir skjema og innhold som beskriver hvilken 
      *                         mimetype den aktuele innholdsposten er av.<br><br>   
     * @param sizeInBytes       Angir strrelsen p fila.<br><br>
     * @param data              Angir selve innholdselementet. -Ingen tegn i dette elementet m kunne forveksles med XML, med mindre de er
      *                         "pakket inne" i hendholdt til xml-spesifiasjonen.<br><br>
     */
    //  * @param encoding          Angir data komprimeringen p innholdselementet. er ikke tatt med da enkoding og data ikke kan forekomme 
    //                                sammen.<br><br>
    public ContentItem( String comment, Catalog catalog, String mediaTypeFN,  String formatFN, 
                        String mimeTypeFN, String sizeInBytes, String input, boolean isData){ 
        this.comment            = comment;
        this.catalog            = catalog;
        this.mediaTypeFN        = mediaTypeFN;
        this.formatFN           = formatFN;
        this.mimeTypeFN         = mimeTypeFN;
        this.notationFN         = notationFN;
        this.sizeInBytes        = sizeInBytes;
        this.contentItemSub     = new ContentItemSub(input, isData);
        
    }
    /**
     * Minste lovlige konstruktoren, alle ContentItem m innholde minst en catalog og data
     * @param catalog           Identifiserer standard vokabular for ContetItem. <br>
     *                          Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                          Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param data              Angir selve innholdselementet. -Ingen tegn i dette elementet m kunne forveksles med XML, med mindre de er
     *                          "pakket inne" i hendholdt til xml-spesifiasjonen.<br><br>
     */
    public ContentItem(Catalog catalog, String input, boolean isData){
        this.catalog    = catalog;
        this.contentItemSub     = new ContentItemSub(input, isData);
    }
    
    /**
     * Metoden returnerer alle contentItem variablene, i xml-format.
     *
     */
    public String getContentItem(){
        String t = "<ContentItem Href=\"" + contentItemUrl + "\">";
        if (comment != null) t = t + "<Comment>" + comment + "</Comment>";
        t = t + "<Catalog>" + catalog.getCatalog() + "</Catalog>";
        t = t + "<MediaType FormalName=\"" + mediaTypeFN + "\" Scheme=\"" + mediaTypeScheme + "\"/>";
        t = t + "<Format FormalName=\"" +  formatFN + "\" Scheme=\"" + formatScheme + "\"/>";
        t = t + "<MimeType FormalName=\"" +  mimeTypeFN + "\" Scheme=\"" + mimeTypeScheme + "\"/>";
        t = t + "<Notation FormalName=\"" +  notationFN + "\" Scheme=\"" + notationScheme + "\"/>";
        t = t + "<Characteristics>";
        t = t + "<SizeInBytes>" + sizeInBytes + "</SizeInBytes>";
        t = t + "<Property FormalName=\"" + propertyFN + "\" Value=\"" + propertyValue + "\"/>";
        t = t + "</Characteristics>";
        t = t + contentItemSub.toString();
        t = t + "</ContentItem>";
        return t;
    }
    
    /**
     * Metoden returnerer alle contentItem variablene, i xml-format.
     */
    public String getNewContentItem(){
        StringBuffer sb = new StringBuffer();
        sb.append("<ContentItem Href=\"");
        sb.append(contentItemUrl);
        sb.append("\">");
        if (comment != null) {
            sb.append("<Comment>");
            sb.append(comment);
            sb.append("</Comment>");
        }
        sb.append("<Catalog>");
        sb.append(catalog.getCatalog());
        sb.append("</Catalog>");
        sb.append("<MediaType FormalName=\"");
        sb.append(mediaTypeFN);
        sb.append("\" Scheme=\"");
        sb.append(mediaTypeScheme);
        sb.append("\"/>");
        sb.append("<Format FormalName=\"");
        sb.append(formatFN);
        sb.append("\" Scheme=\"");
        sb.append(formatScheme);
        sb.append("\"/>");
        sb.append("<MimeType FormalName=\"");
        sb.append(mimeTypeFN);
        sb.append("\" Scheme=\"");
        sb.append(mimeTypeScheme);
        sb.append("\"/>");
        sb.append("<Notation FormalName=\"");
        sb.append(notationFN);
        sb.append("\" Scheme=\"");
        sb.append(notationScheme);
        sb.append("\"/>");
        sb.append("<Characteristics>");
        sb.append("<SizeInBytes>");
        sb.append(sizeInBytes);
        sb.append("</SizeInBytes>");
        sb.append("<Property FormalName=\"");
        sb.append(propertyFN);
        sb.append("\" Value=\"");
        sb.append(propertyValue);
        sb.append("\"/>");
        sb.append("</Characteristics>");
        sb.append(contentItemSub.toString());
        sb.append("</ContentItem>");
        return sb.toString();
    }
    
    /******************************************************
     **************Private class contentItemSub************
     ******************************************************/
    private class ContentItemSub{
        private String encoding         = null;
        private String data             = null;
        
        public ContentItemSub(String input, boolean isData){
            if (isData) this.data = input;
            else this.encoding = input;
        }
        
        public String toString(){
            //hva skjer hvis data = null?
            if (encoding != null) return "<Encoding Notation=\"" + encoding + "\"/>";
            if (data !=null){
                System.out.println("data er lik null");
                return "";
            } 
            else return "<DataContent>" + data + "</DataContent>";
        }
    }
}