package no.ingeberg.NewsML;
import java.util.*;

/**
 * Identification.java<br>
 * Klassen innholder informasjon som tar sikte p  globalt identifisere en nyhetspost. Den innholder bde formelle identifkasjons
 * verdier, og en rekke betegnelses felt som gjr det lettere for mennesker  identifisere nyhetspostene
 * (disse er ikke ment  vre globalt unike).
 * Klassen har metoder for  sette ulike under element og en metode for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 * Identification elementet er oblikatorisk og skal forekomme kun 1 gang.
 * @author Marit Ingeberg
 */
public class Identification {
    private NewsIdentifier newsIdentifier   = null; //1
    private String nameLabel                = null; //0-1
    private String dateLabel                = null; //0-1 Skal inne holde en string representasjon av dato'en.
    private int labelSetNr                  = 0; 
    //LabelSet er en kombinasjon av stringene LabelType og Label Text, som alltid forekommer parvis sammen, kan ha 0-N forekomster.
    private ArrayList labelSet              = null;
    
    
    /**
     *Oppretter en ny istans av catalog, i denne konstruktoren er alle under elementene tatt med
     * @param newsIdentifier  Tilfrer nyhetsposten globalte unike identifikatorer.<br><br>
     * @param nameLabel       En string som kan identifiser nyhetspostens navn for mennesker. Kan ikke lages p flere sprk.<br><br>
     * @param dateLabel       En string som innholder en representerer dato, p det formatet som er bekvemt for brukeren.<br><br>
     * @param labelType       Frivillig og menneskeleselig felt som sammen med LabelText utgjr "Label, som brukes til  beskrive ting 
     *                        som ikke dekkes av det vrige punktene.  Innholder en verdi fra et kontrollert vokabular.<br><br>
     * @param labelText       Frivillig og menneskeleselig felt som sammen med LabelType utgjr "Label, som brukes til  beskrive ting som 
     *                        ikke dekkes av det vrige punktene. Innholder fri tekst.<br><br>
     */
    public Identification(NewsIdentifier newsIdentifier, String nameLabel, String dateLabel, String labelType, String labelText){
        this.newsIdentifier = newsIdentifier;
        this.nameLabel      = nameLabel;
        this.dateLabel      = dateLabel;
        this.labelSet        = new ArrayList();
        this.labelSet.add(new LabelSet(labelType, labelText));
    }
    
    
    /**
     * Minste lovlige konstruktoren, alle Identification m innholde minst en NewsIdentifier.
     * @param newsIdentifier  Tilfrer nyhetsposten globalte unike identifikatorer.
     */
    public Identification(NewsIdentifier newsIdentifier){
        this.newsIdentifier = newsIdentifier;
    }
    
    /**
     * Metoden legger til "ekstra" label element. Kan gjentaes s ofte som nskelig.
     * @param labelType       Frivillig og menneskeleselig felt som sammen med LabelText utgjr "Label, som brukes til  beskrive ting som 
     *                        ikke dekkes av det vrige punktene.  Innholder en verdi fra et kontrollert vokabular.<br><br>
     * @param labelText       Frivillig og menneskeleselig felt som sammen med LabelType utgjr "Label, som brukes til  beskrive ting som 
     *                        ikke dekkes av det vrige punktene. Innholder fri tekst.
     */
    public void addLabel(String labelType, String labelText){
        this.labelSet.add(new LabelSet(labelType, labelText));
        
    }
    /**
     * Metoden setter nameLabel elementet.
     * @param nameLabel       En string som kan identifiser nyhetspostens navn for mennesker. Kan ikke lages p flere sprk.
     */
    public void setNameLabel(String nameLabel) { this.nameLabel = nameLabel; }
    
    /**
     *Metoden setter dataLabel elementet.
     * @param dateLabel       En string som innholder en representerer dato, p det formatet som er bekvemt for brukeren.
     */
    public void setDateLabel(String dateLabel) { this.dateLabel = dateLabel; }
    
    /**
     * Metoden returnerer alle getIdentification variablene, i xml-format.
     *
     */
    public String getIdentification() {
                /*<Identification>
                    <NewsIdentifier>
                        <Providerld>naaf.no</Providerld>
                        <DateId>20031120</DateId>
                        <NewsItemId>20031120.01.</NewsItemId>
                        <RevisionId PreviousRevision="0" Update="U">1</RevisionId>
                        <PublicIdentifier>urn:NewsMl:naaf.no:20021120.01.</PublicIdentifier>
                    </NewsIdentifier>
                    <NameLabel>Pollenvasling</NameLabel>
                  </Identification>*/
        String t = "<Identification>";
        t = t + newsIdentifier.getNewsIdentifier();
        if (nameLabel != null){ t = t + "<NameLabel>"+nameLabel+"</NameLabel>";}
        if (dateLabel != null){t = t + "<DateLabel>" + dateLabel + "</DateLabel>"; }
        if (labelSet != null){
            Iterator i = labelSet.iterator();
            while (i.hasNext()){
                t = t + i.next().toString();
            }
        }
        t = t + "</Identification>";
        return t;
    }
    
    /////////////////////////////////////////////////
    //////////////// PRIVATE CLASS LabelSet//////////
    /////////////////////////////////////////////////
    /**
     * Klassen legger til "ekstra" label element.
     */
    private class LabelSet {
        String labelType = null;
        String labelText = null;
        
        /**
         * Metoden legger til "ekstra" label element. Kan gjentaes s ofte som nskelig.
         * @param labelType       Frivillig og menneskeleselig felt som sammen med LabelText utgjr "Label, som brukes til  beskrive ting 
         *                        som ikke dekkes av det vrige punktene.  Innholder en verdi fra et kontrollert vokabular.<br><br>
         * @param labelText       Frivillig og menneskeleselig felt som sammen med LabelType utgjr "Label, som brukes til  beskrive ting
         *                        som ikke dekkes av det vrige punktene. Innholder fri tekst.
         */
        public LabelSet(String labelType, String labelText){
            this.labelType = labelType;
            this.labelText = labelText;
        }
        
        /**Metoden skriver ut label elementet formatert som xml*/
        public String toString() {
            return new String("<Label> <LabelType FormalName=\"" + labelType + "\"></LabelType> <LabelText>" 
                                + labelText + "</LabelText> </Label>");
        }
    }
}