package no.ingeberg.NewsML;
import java.net.*;

/**
 * NewsComponentSub.java <br>
 * Klassen tar seg av hndteringa av
 * NewsItem | NewsItemRef | NewsComponent | ContentItem elementene der kun en
 * var de skal forekomme. Klassen innholder og metoder for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */
public class NewsComponentSub {
    private NewsItem newsItem                       = null;
    private URL newsItemRef                         = null;
    private NewsComponent newsComponent             = null;
    private ContentItem contentItem                 = null;
    
    /*
     *Oppretter en ny instans av NewsComponentSub, i denne konstruktoren er det kun
     *NewsItem som er tatt med.
     *@param newsItem  Innholder selve nyhetsobjektet som tilfres identifikasjon og forvaltnings metadata
     *
     */
    public NewsComponentSub(NewsItem newsItem){
        this.newsItem       = newsItem;
    }
    
   /*
    *Oppretter en ny instans av NewsComponentSub, i denne konstruktoren er det kun
    *newsItemRef som er tatt med.
    *@param newsItemRef   Innholder en peker til et newsML objekt som er ment  erstatte NewsItemRef elementet.
    */
    public NewsComponentSub(URL newsItemRef){
        this.newsItemRef    = newsItemRef;
    }
    
   /*
    *Oppretter en ny instans av NewsComponentSub, i denne konstruktoren er det kun
    *NewsComponent som er tatt med.
    *@param NewsComponent  Innholder et newsComponent objekt som er et beholder objekt
    * som grupperer sammen ulike nyhetsobjekt (nyhetsposter, nyhetspost referanser, og nyhetskomponenter) og innholds element.
    */
    public NewsComponentSub(NewsComponent newsComponent){
        this.newsComponent  = newsComponent;
    }
    
    /*
     *Oppretter en ny instans av NewsComponentSub, i denne konstruktoren er det kun
     *ContentItem som er tatt med.
     *@param ContentItem Inneholder et nyhetsobjekt som identoifiserer innhold (slik som tekst, bilder, video, lyd etc) ment 
               for presentasjon for mennesker.
     */
    public NewsComponentSub(ContentItem contentItem){
        this.contentItem   = contentItem;
    }
    
    /**
     * Metoden returnerer det elementet som er aktuelt, , i xml-format. Hvilket
     * dette er avheninger av hvilke parametere som er sendt med inn.
     *
     */
    public String getNewsComponentSub(){
        String t = "";
        if (newsItem != null){
            t = newsItem.getNewsItem();
            return t;
        }
        else if (newsItemRef != null){
            t = "<NewsItemRef Euid=\"" + newsItemRef + "\"/>";//?????
            return t;
        }
        else if (newsComponent != null){
            t = newsComponent.getNewsComponent();
            return t;
        }
        else if (contentItem != null){
            t = contentItem.getContentItem();
            return t;
        }
        //throw new Exception();
        return t; //Denne skal i prinsippet aldri inntreffe.
    }
}
