package no.ingeberg.NewsML;
import java.text.*;
import java.util.*;
/**
 * NewsEnvelope.java<br>
 * Elementet innholder informasjon om hvordan NewsML dokumentet blir brukt innen organisasjonens workflow
 * eller kontraktsmessige forhold mellom nyhetsutgiver og mottaker. Denne klassen innholder kun
 * dato/tid for overfring og prioritet, og innholder metode for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class NewsEnvelope {
    private String dateAndTime = null; //1
    private String priority = null; //0-1
    
    /**
     * Oppretter en ny instans av NewsEnvelope.
     * @param dateAndTime  innholder dato + ev. valgfri tid, for overfring. <br>
     * Dette er en ISO 8601:2000(E)format (MMDD), frivillig etterfulgt av bokstaven T + lokal tid skrevet p TTMMSS formatet,
     * inneholder ogs en frivillig + eller  tegn fulgt av TTMM forskjellen mellom lokaltid og
     * GMT (Greenwich mean time). <br>
     * Er lokal tid den samme som GMT, alts +0000 kan bokstaven Z brukes som et alternativ.<br><br>
     * @param priority innholder en indikasjon p prioriteten til nyhetsposten. Betydningen og tillatte verdier er hentet et kontrollerte 
     * vokabularet.
     */
    
    
     public NewsEnvelope() {
        this.dateAndTime = dateAndTime;
        //dateAndTime = getDateAndTime();
    }

    public NewsEnvelope(String dateAndTime, String priority) {
        this.priority = priority;
        this.dateAndTime = dateAndTime;
        //dateAndTime = getDateAndTime();
    }
    
    /*
     *Lager minste mulige instans av NewsEnvelope
    * @param dateAndTime  innholder dato + ev. valgfri tid, for overfring. <br>
     * Dette er en ISO 8601:2000(E)format (MMDD), frivillig etterfulgt av bokstaven T + lokal tid skrevet p TTMMSS formatet,
     * inneholder ogs en frivillig + eller  tegn fulgt av TTMM forskjellen mellom lokaltid og
     * GMT (Greenwich mean time). <br>
     * Er lokal tid den samme som GMT, alts +0000 kan bokstaven Z brukes som et alternativ.<br><br>
     */
    public NewsEnvelope(String dateAndTime) {
        this.dateAndTime = dateAndTime;
        //dateAndTime = getDateAndTime();
    }
    /*
     *Metoden returnerer DateAndTime p rett format.
    private String getDateAndTime(){
        SimpleDateFormat datoForm=new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat tidsForm=new SimpleDateFormat("HHmm");
        Date dato = new Date();
        return datoForm.format(dato) + "T" + tidsForm.format(dato) + "+0" +
        new Integer(TimeZone.getDefault().getDSTSavings()/36000).toString();
    }
     */
    
   
    
    /**
     * Metoden returnerer alle NewsEnvelope variablene, i xml-format.
     *
     */
    public String getNewsEnvelope(){
        String t = "<NewsEnvelope> <DateAndTime>" + dateAndTime + "</DateAndTime>";
        if (priority != null) t = t + "<Priority FormalName=\"" + priority + "\"/>";
        t = t + "</NewsEnvelope>";
        return t;
    }
}
