package no.ingeberg.NewsML;

/**
 * NewsIdentifier.java <br>
 * Klassen innholder informasjon som tar sikte p  globalt identifisere en nyhetspost.
 * Klassen har metoder for  sette ulike under element og en metode for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 * Elementet er oblikatorisk og skal forekomme kun 1 gang.
 *
 * @author Marit Ingeberg
 */
public class NewsIdentifier {
    private String providerId = null; // 1
    private String dateId = null; // 1
    private String newsItemId = null; // 1
    private String revisionId = null; // 1
    private String previousRevision = null; // 1
    private String update = null; // 1
    private String publicIdentifier = null; //1
    
    /**
     *Oppretter en ny istans av catalog, i denne konstruktoren er alle under
     *elementene tatt med, og alle er obligatoriske.
     * @param providerId        Elementet m vre et Internett domene navn som er eid av provider ved den aktuelle datoen angitt i dato-id 
     *                          elementet, eller det kan vre et navn provider henter fra et kontrollert vokabular identifisert ved en URN, 
     *                          som blir spesifisert i vokabular attributtet.<br><br>
     * @param dateId            Innholder en dato, skrevet i ISO 8601 Basic Format (r, mned, dato). M vre samme gjennom alle revisjonen av 
     *                          den aktuelle nyhetsposten. Den vil ikke representere datoen for utgivelse av nvrende revisjon.<br><br>
     * @param newsItemId        Innholder en identifikator for en nyhetspost. Kombinasjonen av nyhetspost-id og dato-id m vre unik blant 
     *                          nyhetsposter som er fra samme provider. Med disse begrensningene, kan en nyhetspost-id vre p den formen 
     *                          provider nsker.<br><br>
     * @param revisionId        Elementet skal innholde et positivt heltall som forteller hvilken revisjonsnummer en nyhetspost er. Den nyeste 
     *                          instansen av en nyhetspost m alltid ha det hyeste revisjons-iden.<br><br>
     * @param previousRevision  Elementet settes i forhold til revisionID. revisionID minus 1. Elementet er obligatorisk.<br><br>
     * @param update            Hvis en nyhetspost innholder et eller flere oppdaterte element m en sette et update attributt til U. 
     *                          Hvis nyhetsposten kun bestr av nyhets forvaltningsdata m dette attributtet settes til A. Hvis ikke noe av 
     *                          dette er tilfelle settes attributtet til N.<br><br>
     * @param publicIdentifier  En streng, p flgende form: "urn:newsml:{ProviderId}:{DateId}:{NewsItemId}:{RevisionId}{RevisionId@Update}"
     *                          <br><br>
     */
    public NewsIdentifier(String providerId, String dateId, String newsItemId,String revisionId, String previousRevision, String update, 
                            String publicIdentifier){
        this.providerId = providerId;
        this.dateId = dateId;
        this.newsItemId = newsItemId;
        this.revisionId = revisionId;
        this.previousRevision = previousRevision;
        this.update = update;
        this.publicIdentifier = publicIdentifier;
    }
    
    
    /*Set metoder som kan brukes hvis en skal fullfre kode for  endre innhold i newsML dokumentet
    public void setProviderID(String providerId) { this.providerId = providerId; }
    public void setDateID(String dateId) {this.dateId = dateId; }
    public void setNewsItemID(String newsItemId) { this.newsItemId = newsItemId; }
    public void setRevisionID(String ID) { this.revisionId = ID; }
    public void setPublicIdentifier(String publicIdentifier) { this.publicIdentifier = publicIdentifier; }
     */
    
    /**
     * Metoden returnerer alle NewsIdentifer variablene, i xml-format.
     *
     */
    public String getNewsIdentifier(){
        String t ="<NewsIdentifier>";
        t = t + "<ProviderId>"+providerId+"</ProviderId>";
        t = t + "<DateId>"+dateId+"</DateId>";
        t = t + "<NewsItemId>"+newsItemId+"</NewsItemId>";
        t = t + "<RevisionId PreviousRevision=\"" + previousRevision + "\" Update=\"" + update + "\">"+revisionId+"</RevisionId>";
        t = t + "<PublicIdentifier>"+publicIdentifier+"</PublicIdentifier>";
        t = t + "</NewsIdentifier>";
        return t;
    }
    
}