package no.ingeberg.NewsML;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * NewsItem.java<br><br>
 *
 * Klassen setter sammen all informasjonen som skal til for  f en gyldig nyhetspost.
 * Klassen i seg selv setter ingen annen inforamsjon en et "comment" element, men kaller metoder
 * i andre klasser, og innholder metoder for  sette alle nskelige nyhetspost -komponenter,
 * og en metode for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author  Marit Ingeberg
 */

public class NewsItem {
    private int commentNr                 = 0; //comment er en stiring som kan forekomme 0-N ganger
    private ArrayList comment             = null;
    private Catalog catalog               = null; //0-1
    private Identification identification = null; //1
    private NewsManagement newsManagement = null; //1
    private NewsItemSub newsItemSub       = null; //0-1
    
    /** Oppretter en ny instans av newsItem
     * @param comment       En string som innholder frivillig kommentar til nyhetsposten. <br><br>
     * @param catalog       Identifiserer standard vokabular for newsItem. <br>
     *                      Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                      Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param identification Innholder informasjon som skal identifisere nyhetsposten globalt.<br><br>
     * @param newsManagement Innholder informasjon som er relevant for forvaltning av nyhetsposten.<br><br>
     * @param newsItemSub   Kaller et av elementete <br>
     * NewsItem | NewsItemRef | NewsComponent | ContentItem <br>
     * - der kun et av elementene skal forekomme. Legger inn informasjon avhening av hvilket element som blir hentet ut.
     */
    public NewsItem(String comment, Catalog catalog, Identification identification,
    NewsManagement newsManagement, NewsItemSub newsItemSub) {
        this.comment = new ArrayList();
        this.catalog = catalog;
        this.identification = identification;
        this.newsManagement = newsManagement;
        this.newsItemSub = newsItemSub;
        this.comment.add(comment);
    }
    
     /** Oppretter en ny instans av newsItem
     * @param identification Innholder informasjon som skal identifisere nyhetsposten globalt.<br><br>
     * @param newsManagement Innholder informasjon som er relevant for forvaltning av nyhetsposten.<br><br>
     * @param newsItemSub   Kaller et av elementete <br>
     * NewsItem | NewsItemRef | NewsComponent | ContentItem <br>
     * - der kun et av elementene skal forekomme. Legger inn informasjon avhening av hvilket element som blir hentet ut.
     */
    public NewsItem(Identification identification, NewsManagement newsManagement,
            NewsItemSub newsItemSub) {
        this.identification = identification;
        this.newsManagement = newsManagement;
        this.newsItemSub = newsItemSub;
    }
    
    /*minste lovlige konstruktor, innholder kun identification og newsManagement
     *@param identification Innholder informasjon som skal identifisere nyhetsposten globalt.<br><br>
     * @param newsManagement Innholder informasjon som er relevant for forvaltning av nyhetsposten.
     */
    public NewsItem(Identification identification, NewsManagement newsManagement) {
        this.identification = identification;
        this.newsManagement = newsManagement;
    }
    
    /**Metoden legger til "ekstra" comment element. Kan gjentaes s ofte som nskelig.
     * @param comment       En string som innholder frivillig kommentar til nyhetsposten. <br><br>
     */
    public void addComment(String comment){
        this.comment.add(comment);
    }
    
    /**Metoden setter catalog elementet.
     * @param catalog       Identifiserer standard vokabular for newsItem. <br>
     *                      Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                      Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     */
    public void setCatalog(Catalog catalog){
        this.catalog = catalog;
    }
    
    /**Metoden setter identification elementet.
     * @param identification Innholder informasjon som skal identifisere nyhetsposten globalt.
     */
    public void setIdentification(Identification identification){
        this.identification = identification;
    }
    
    /**Metoden setter newsManagement elementet.
     * @param newsManagement Innholder informasjon som er relevant for forvaltning av nyhetsposten.
     */
    public void  setNewsManagement(NewsManagement newsManagement){
        this.newsManagement = newsManagement;
    }
    
    /**Metoden setter newsItemSub elementet.
     * @param newsItemSub   Kaller et av elementete NewsItem | NewsItemRef | NewsComponent | ContentItem <br>
     * der kun et av elementene skal forekomme. Legger inn informasjon avhening av hvilket element som blir hentet ut.
     */
    public void setNewsItemSub(NewsItemSub newsItemSub){
        this.newsItemSub = newsItemSub;
    }
    
     /**
     * Metoden returnerer alle NewsItem elementene, i xml-format.
     *
     */
    public String getNewsItem(){
        String t = "<NewsItem>";
        if (comment != null){
            Iterator i = comment.iterator();
            while (i.hasNext()){
                t = t + "<Comment>" + i.next().toString() + "</Comment>";
            }
        }
        if (catalog != null) {t = t + catalog.getCatalog();}
        t = t + identification.getIdentification();
        t = t + newsManagement.getNewsManagement();
        if (newsItemSub != null){t = t + newsItemSub.getNewsItemSub();}
        t = t + "</NewsItem>";
        return t;
    }    
}
