package no.ingeberg.NewsML;
import java.net.*;
import java.io.*;
import java.util.*;
/**
 * NewsItemSub.java<br>
 * Klassen hndterer innkomne data under "NewsItemSub", som er en samling av
 * elementene:<br>
 * "NewsComponent" / "Update" / "TopicSet"<br>
 * I denne gruppen er det kun et av elementene som skal forekomme, i NewsItem.
 * Elementet er frivillig, eller den kan forekomme kun 1 gang.
 * Klassen jobber direkte mot tre klasser: "NewsComponent", "Update", og
 * "TopicSet". Klassen innholder metoder for  hente ut informasjon fra disse
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class NewsItemSub {
    private NewsComponent newsComponent = null;
    private int updateNr = 0;
    private ArrayList update = null;
    private TopicSet topicSet = null;
    
    /**
     *Oppretter en ny instans av NewsItemSub, i denne konstruktoren er det kun
     *NewsComponent som er tatt med.
     * @param NewsComponent  Innholder et newsComponent objekt som er et beholder objekt
     * som grupperer sammen ulike nyhetsobjekt (nyhetsposter, nyhetspost referanser, og nyhetskomponenter) og innholds element.
     */
    public NewsItemSub(NewsComponent newsComponent){
        this.newsComponent = newsComponent;
    }
    
    /**Oppretter en ny instans av NewsItemSub, i denne konstruktoren er det kun
     *"update" som er tatt med.
     * @param    Angir om nyhetsposten innholder et eller flere oppdaterte element, verdi U. 
     *          Hvis nyhetsposten kun bestr av nyhets forvaltningsdata m dette attributtet settes til A. 
     * Hvis ikke noe av dette er tilfelle settes attributtet til N.
     */
    public NewsItemSub(String update){
        //this.update = update;
        this.update = new ArrayList();
        this.update.add(update);
    }
    
    /*
     *Oppretter en ny instans av NewsItemSub, i denne konstruktoren er det kun
     *Topicset som er tatt med.
     * @topicSet Angir emnet til deler eller hele nyhetsposter.
     */
    public NewsItemSub(TopicSet topicSet){
        this.topicSet = topicSet;
    }
    
    /**
     *Legger til et update element
     *@param . Angir om nyhetsposten innholder et eller flere oppdaterte element, verdi U. Hvis nyhetsposten kun bestr av 
     *          nyhetsforvaltningsdata m dette attributtet settes til A. 
     * Hvis ikke noe av dette er tilfelle settes attributtet til N.
     */
    public void addUpdate(String update){
        this.update.add(update);
    }
    
    /**
     * Metoden returnerer det elementet som er aktuelt, , i xml-format. Hvilket
     * dette er avheninger av hvilke parametere som er sendt med inn.
     *
     */
    public String getNewsItemSub(){
        String t = null;
            if (newsComponent != null){
            t = newsComponent.getNewsComponent();
            return t;
        }
        if (update != null){
            Iterator i = update.iterator();
            t = "<Update Euid=\"" + ((String)i.next()).toString() + "\"/>";
            while (i.hasNext()){
                t = t + "<Update Euid=\"" + ((String)i.next()).toString() + "\"/>";
            }
            return t;
        }
        if (topicSet != null){
            t = t + topicSet.getTopicSet();
            return t;
        }
        //throw new Exception();
        return t; //Denne skal i prinsippet aldri inntreffe.
    }
}
