package no.ingeberg.NewsML;
import java.util.*;
/**
 * NewsLines.java <br>
 * Klassen hndterer innkomne data under "NewsLines" elementet,
 * som angir karakteristiske egenskaper ved nyhetsobjektet.
 * Disse er ment  tilfre en representasjon av metadata som er lett tilgjenglig for mennesker.
 * Som regel vil disse angi nyhetslinene som blir brukt i nyhetsinnlegget.
 * Klassen innholder metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class NewsLines {
    private ArrayList headLineMap   = null;
    private int byLineNr            = 0; //byline er en sting som kan forekomme 0-N ganger
    private ArrayList byLine        = null;
    private int dateLineNr          = 0; //dateLine er en sting som kan forekomme 0-N ganger
    private ArrayList dateLine      = null;
    private int creditLineNr        = 0; //creditline er en sting som kan forekomme 0-N ganger
    private ArrayList creditLine    = null;
    private int copyrightLineNr     = 0; //copyrighyLine er en sting som kan forekomme 0-N ganger
    private ArrayList copyrightLine = null;
    private int rightsLineNr        = 0; //rightsLine er en sting som kan forekomme 0-N ganger
    private ArrayList rightsLine    = null;
    private int seriesLineNr        = 0; //seriesLine er en sting som kan forekomme 0-N ganger
    private ArrayList seriesLine    = null;
    private int slugLineNr          = 0; //sligline er en sting som kan forekomme 0-N ganger
    private ArrayList slugLine      = null;
    private int keywordLineNr       = 0; //keywordLine er en sting som kan forekomme 0-N ganger
    private ArrayList keywordLine   = null;
    private int newsLineMapNr       = 0; 
    //newslineMap er en kombinasjon av NewsLine type og text, der type -> 1 text ->1-N og kombinasjonene som en helehet kan forekomme 0-N ganger
    private ArrayList newsLineMap   = null;
    
    /**
     *Oppretter en ny instans av NewsLines, i denne konstruktoren er alle under elementene tatt med
     * @param headLine          angir visbare overskriften.<br><br>
     * @param subHeadLine       angir visbare under-overskrift .<br><br>
     * @param byLine            angir i naturlig sprk informasjon om journalist/forfatter.<br><br>
     * @param dateLine          angir i naturlig sprk informasjon om dato og/eller stedet for nr nyhetskomponenten ble laget.<br><br>
     * @param creditLine        angir i naturlig sprk kreditt informasjon.<br><br>
     * @param copyrightLine     angir i naturlig sprk informasjon om copyright. Dette forteller hvem som eier objektet.<br><br>
     * @param rightsLine        angir den visbare versjonen av rettighets informasjonen. Alts om hvem som har lov  bruke objektet og hvordan 
     *                          det da skal brukes.<br><br>
     * @param seriesLine        angir den visbare informasjonen om objektets plass i en ev. serie.<br><br>
     * @param slugLine  	angir en streng av tekst (ev. med en hyperlink) som brukes til den visbare slug-line. Hva som legges i 
     *                          slug-line er opptil den enkelte redaksjon.<br><br>
     * @param keywordLine       angir den eller de visbare nkkelordene som er relevant for nyhetsobjektet.<br><br>
     * @param newsLineType	angir en brukerdefinert nyhetslinjetype, der verdiene hentes fra et kontrollert vokabular.  
     *                          Utgjr sammen med "newsLineText" - "newsLine"<br><br>
     * @param newsLineText	angir innhold til den definerte NewsLineType.  Utgjr sammen med "newsLineText" - "newsLine".<br><br>
     */
    public NewsLines(String headLine, String subHeadLine, String byLine, String dateLine, String creditLine, String copyrightLine, 
                        String  rightsLine, String seriesLine, String slugLine, String keywordLine, String newsLineType, ArrayList newsLineText){
        this.headLineMap     = new ArrayList();
        this.byLine          = new ArrayList();
        this.dateLine        = new ArrayList();
        this.creditLine      = new ArrayList();
        this.copyrightLine   = new ArrayList();
        this.rightsLine      = new ArrayList();
        this.seriesLine      = new ArrayList();
        this.slugLine        = new ArrayList();
        this.keywordLine     = new ArrayList();
        this.newsLineMap     = new ArrayList();
        
        addHeadLineMap(headLine, subHeadLine);
        this.byLine.add(byLine);
        this.dateLine.add(dateLine);
        this.creditLine.add(creditLine);
        this.copyrightLine.add(copyrightLine);
        this.rightsLine.add(rightsLine);
        this.seriesLine.add(seriesLine);
        this.slugLine.add(slugLine);
        this.keywordLine.add(keywordLine);
        addNewsLineMap(newsLineType, newsLineText);
    }
    
    /**
     *Oppretter en ny instans av NewsLines. Flgende element er tatt med.
     * @param headLine          angir visbare overskriften.<br><br>
     * @param subHeadLine       angir visbare under-overskrift .<br><br>
     * @param byLine            angir i naturlig sprk informasjon om journalist/forfatter.<br><br>
     * @param creditLine        angir i naturlig sprk kreditt informasjon.<br><br>
     * @param rightsLine        angir den visbare versjonen av rettighets informasjonen. Alts om hvem som har lov  bruke objektet og 
     *                          hvordan det da skal brukes.<br><br>
     * @param seriesLine        angir den visbare informasjonen om objektets plass i en ev. serie.<br><br>
     * @param slugLine  	angir en streng av tekst (ev. med en hyperlink) som brukes til den visbare slug-line. Hva som legges i 
     *                          slug-line er opptil den enkelte redaksjon.<br><br>
     * @param keywordLine       angir den eller de visbare nkkelordene som er relevant for nyhetsobjektet.<br><br>
     */
    public NewsLines(String headLine, String subHeadLine, String byLine, String creditLine, 
            String rightsLine, String seriesLine, String slugLine, String keywordLine){
        this.headLineMap     = new ArrayList();
        this.byLine          = new ArrayList();    
        this.creditLine      = new ArrayList();        
        this.rightsLine      = new ArrayList();
        this.seriesLine      = new ArrayList();
        this.slugLine        = new ArrayList();
        this.keywordLine     = new ArrayList();
        
        addHeadLineMap(headLine, subHeadLine);
        this.byLine.add(byLine);
        this.creditLine.add(creditLine);
        this.rightsLine.add(rightsLine);
        this.seriesLine.add(seriesLine);
        this.slugLine.add(slugLine);
        this.keywordLine.add(keywordLine);
    }
    /**
     * Minste lovlige konstruktoren,  -som er tom.
     */
    public NewsLines(){
        //det er lov  kalle en tom NewsLine.
    }
    
    
    
    /** Metoden legger til et "ekstra" headline og et subheadline element. Kan gjentaes.
     * @param headLine          angir visbare overskriften.<br><br>
     * @param subHeadLine       angir visbare under-overskrift .
     */
    public void addHeadLineMap(String headLine, String subHeadLine){
        this.headLineMap.add(new HeadLineMap(headLine, subHeadLine));
    }
    
    /** Metoden legger til et "ekstra" headline uten et subheadline element. Kan gjentaes.
     * @param headLine          angir visbare overskriften.<br><br>
     */
    public void addHeadLineMap(String headLine){
        this.headLineMap.add(new HeadLineMap(headLine));
    }
    
    
    /** Metoden legger til "ekstra" byLine element. Kan gjentaes s ofte som nskelig.
     * @param byLine angir i naturlig sprk informasjon om journalist/forfatter.
     */
    public void addByLine(String byLine){
        this.byLine.add(byLine);
    }
    
    /** Metoden legger til "ekstra" dateLine element. Kan gjentaes s ofte som nskelig.
     * @param dateLine angir i naturlig sprk informasjon om dato og/eller stedet for nr nyhetskomponenten ble laget.
     */
    public void addDateLine(String dateLine){
        this.dateLine.add(dateLine);
    }
    
    /**Metoden legger til "ekstra" creditLine element. Kan gjentaes s ofte som nskelig.
     * @param creditLine angir i naturlig sprk kreditt informasjon.
     */
    public void addCreditLine(String creditLine){
        this.creditLine.add(creditLine);
    }
    
    /**Metoden legger til "ekstra" copyrightLine element. Kan gjentaes s ofte som nskelig.
     * @param copyrightLine angir i naturlig sprk informasjon om copyright. Dette forteller hvem som eier objektet.
     */
    public void addCopyrightLine(String copyrightLine){
        this.copyrightLine.add(copyrightLine);
    }
    
    /**Metoden legger til "ekstra" rightsLine element. Kan gjentaes s ofte som nskelig.
     * @param rightsLine    angir den visbare versjonen av rettighets informasjonen. Alts om hvem som har lov  bruke objektet og hvordan det 
     *                      da skal brukes.
     */
    public void addRightsLine(String rightsLine){
        this.rightsLine.add(rightsLine);
    }
    
    /**Metoden legger til "ekstra" seriesLine element. Kan gjentaes s ofte som nskelig.
     * @param seriesLine angir den visbare informasjonen om objektets plass i en ev. serie.
     */
    public void addSeriesLine(String seriesLine){
        this.seriesLine.add(seriesLine);
    }
    
    /**Metoden legger til "ekstra" slugLine element. Kan gjentaes s ofte som nskelig.
     * @param slugLine      angir en streng av tekst (ev. med en hyperlink) som brukes til den visbare slug-line. Hva som legges i slug-line er 
     *                      opptil den enkelte redaksjon.
     */
    public void addSlugLine(String slugLine){
        this.slugLine.add(slugLine);
    }
    
    /**Metoden legger til "ekstra" keywordLine element. Kan gjentaes s ofte som nskelig.
     * @param keywordLine angir den eller de visbare nkkelordene som er relevant for nyhetsobjektet.
     */
    public void addKeywordLine(String keywordLine){
        this.keywordLine.add(keywordLine);
    }
    
    /**Metoden legger til "ekstra" newsLine element. Kan gjentaes s ofte som nskelig.
     * @param newsLineType angir en brukerdefinert nyhetslinjetype, der verdiene hentes fra et kontrollert vokabular.  Utgjr sammen med 
     *                      "newsLineText" - "newsLine"<br><br>
     * @param newsLineText angir innhold til den definerte NewsLineType.  Utgjr sammen med "newsLineText" - "newsLine".
     */
    public void addNewsLineMap(String newsLineType, ArrayList newsLineText){
        this.newsLineMap.add(new NewsLineMap(newsLineType, newsLineText));
    }
    
    
    /**
     * Metoden returnerer alle "NewsLines" variablene, i xml-format.
     *
     */
    public String getNewsLines() {
         /*
          <NewsLines>
              <HeadLine>Angir visbare overskriften. </HeadLine>
              <SubHeadLine>Angir visbare under-overskrift </SubHeadLine>
              <ByLine>Angir i naturlig sprk informasjon om journalist/forfatter. </ByLine>
              <DateLine>Angir i naturlig sprk informasjon om dato og/eller stedet for nr nyhetskomponenten ble laget. </DateLine>
              <CreditLine> angir i naturlig sprk kreditt informasjon.</CreditLine>
              <CopyrightLine> angir i naturlig sprk informasjon om copyright. Dette forteller hvem som eier objektet.</CopyrightLine>
              <RightsLine>Angir den visbare versjonen av rettighets informasjonen. Alts om hvem som har lov  bruke objektet og hvordan det 
                            da skal brukes.</RightsLine>
              <SeriesLine>Angir den visbare informasjonen om objektets plass i en ev. serie. </SeriesLine>
              <SlugLine> Angir en streng av tekst (ev. med en hyperlink) som brukes til den visbare slug-line. Hva som legges i slug-line er
                            opptil den enkelte redaksjon. </SlugLine>
              <KeywordLine>Angir den eller de visbare nkkelordene som er relevant for nyhetsobjektet. </KeywordLine>
              <NewsLine>
                    <NewsLineType FormalName="element som er en bruker definert nyhetslinjetype, der verdiene hentes fra et kontrollert 
                                vokabular. "/>
                    <NewsLineText>Kan innhold et eller flere NewsLineText elementer, som innholder tekst av den definerte NewsLineType. 
                                Finnes det flere instanser av dette elementet m de tilfres xml:lang attributtet som indikerer hvilket 
                                sprk det er skrevet p. </NewsLineText>
              </NewsLine>
          </NewsLines> */
        
        String t = "<NewsLines>";
        if ( headLineMap != null){
            Iterator i = headLineMap.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        
        if ( byLine != null){
            Iterator i = byLine.iterator();
            while (i.hasNext()) t = t + "<ByLine>" + ((String)i.next()).toString() + "</ByLine>";
        }
        
        if ( dateLine != null){
            Iterator i = dateLine.iterator();
            while (i.hasNext()) t = t + "<DateLine>" + ((String)i.next()).toString() + "</DateLine>";
        }
        
        if (creditLine != null){
            Iterator i = creditLine.iterator();
            while (i.hasNext()){
                t = t + "<CreditLine>" + ((String)i.next()).toString() + "</CreditLine>";
            }
        }
        if ( copyrightLine  != null){
            Iterator i = copyrightLine.iterator();
            while (i.hasNext()){
                t = t + "<CopyrightLine>" + ((String)i.next()).toString() + "</CopyrightLine>";
            }
        }
        
        if ( rightsLine != null){
            Iterator i = rightsLine.iterator();
            while (i.hasNext()){
                t = t + "<RightsLine>" + ((String)i.next()).toString() + "</RightsLine>";
            }
        }
        if ( seriesLine   != null){
            Iterator i = seriesLine.iterator();
            while (i.hasNext()){
                t = t + "<SeriesLine>" + ((String)i.next()).toString() + "</SeriesLine>";
            }
        }
        if ( slugLine != null){
            Iterator i = slugLine.iterator();
            while (i.hasNext()){
                t = t + "<SlugLine>" + ((String)i.next()).toString() + "</SlugLine>";
            }
        }
        if ( keywordLine != null){
            Iterator i = keywordLine.iterator();
            while (i.hasNext()){
                t = t + "<KeywordLine>" + i.next().toString() + "</KeywordLine>";
            }
        }
        
        if ( newsLineMap != null){
            Iterator i = newsLineMap.iterator();
            while (i.hasNext()){
                t = t + i.next().toString();
            }
        }
        t= t + "</NewsLines>";
        return t;
    }
    
    ////////////////////////////////////////////////
    ////////Subclass NewsLinesMap//////////////////
    //////////////////////////////////////////////
    /**
     *Klassen legger inn headLine, og subHeadLine. Og sikrer at SubHeadLine bare er tilstedet hvis headLine eksisterer.
     *Innholder metoder for  hente ut elementete formatert som xml.
     */
    private class HeadLineMap{
        private String headLine         = null;
        private String subHeadLine      = null;
        
        /** Oppretter et minste objekt av HeadLineMap, som innholder headLine og subheadline.
         * * @param headLine angir visbare overskriften.<br><br>
         * @param subHeadLine angir visbare under-overskrift .
         */
        public HeadLineMap(String headLine, String subHeadLine){
            this.headLine = headLine;
            this.subHeadLine = subHeadLine;
        }
        
        /** Oppretter et minste objekt av HeadLineMap, som innholder kun headLine.
         * * @param headLine angir visbare overskriften.<br>
         */
        public HeadLineMap(String headLine){
            this.headLine = headLine;
        }
        
        /**
         *
         */
        public String toString(){
            if (subHeadLine != null)
                return new String("<HeadLine>" + headLine + " </HeadLine> <SubHeadLine>" +subHeadLine + "</SubHeadLine>");
            return new String("<HeadLine>" + headLine + " </HeadLine>");
        }
    }
    
    /////////////////////////////////////////////////
    /////////// PRIVATE CLASS  NewsLineMap///////////
    /////////////////////////////////////////////////
    /**
     * Klassen legger inn newsLine element, som beskriver nyhetslinjer som ikke er spesifisert av de vrige. Kan gjentaes s ofte som nskelig.
     *
     
     */
    private class NewsLineMap {
        String newsLineType     = null;
        ArrayList newsLineText  = null;
        
        /**
         * @param newsLineType  angir en brukerdefinert nyhetslinjetype, der verdiene hentes fra et kontrollert vokabular.  
         *                      Utgjr sammen med "newsLineText" - "newsLine"<br><br>
         * @param newsLineText  angir innhold til den definerte NewsLineType.  Utgjr sammen med "newsLineText" - "newsLine".
         */
        public NewsLineMap(String newsLineType, ArrayList newsLineText) {
            this.newsLineType   = newsLineType;
            this.newsLineText    = newsLineText;
        }
        
        /**
         * Metoden ruturnerer newsLine elementet formatert som xml.
         */
        public String toString() {
            String t = new String();
            t= t + "<NewsLine>";
            t= t + "<NewsLineType FormalName=\"" + newsLineType + "\"/>";
            Iterator i = newsLineText.iterator();
            while (i.hasNext())  t= t + "<NewsLineText>" + i.next().toString() + " </NewsLineText>";
            t= t + "</NewsLine>";
            return t;
            
        }
    }
}
