package no.ingeberg.NewsML;
import java.util.*;
import java.io.*;

public class NewsML {
    private NewsEnvelope newsEnvelope       = null;
    private ArrayList newsItems             = new ArrayList();
    private boolean init                    = false;
    private static String LF                = System.getProperty("line.seperator", new String("\n"));
    private static String SLASH             = System.getProperty("file.seperator", new String("\\"));
    private static String TAB               = System.getProperty("tab.seperator", new String("\t")); 
    
    /** Creates a new instance of NewsML */
    public NewsML(NewsEnvelope newsEnvelope, NewsItem newsItem) {
        if (newsEnvelope != null && newsItem != null) {
            this.newsEnvelope   = newsEnvelope;
            newsItems.add(newsItem);
            init                = true;
        }
    }
    
    /**
     *Legger til en ekstra newsItem.
     */
    public boolean addNewsItem(NewsItem newsItem) {
        if (newsItem != null && init) {
            newsItems.add(newsItem);
            return true;
        }
        return false;
    }
    
    /**
     * Returnerer NewsEnvelope eller null vist objektet ikke er initialisert.
     */
    public NewsEnvelope getNewsEnvelope() {
        if (init) return newsEnvelope;
        else return (NewsEnvelope) null;
    }
    
    /**
     * Returnerer en Iterator over NewsItem eller null vist objektet ikke er initialisert.
     */
    public Iterator getNewsItem() {
        if (init) return newsItems.iterator();
        else return (Iterator) null;
    }
/**
 * Returnerer NewsML dokumentet som en string, kun til test bruk
 */    
    public String toString(){
        return writeNewsML("dtd test");
    }
    
     /**
     * Tar inn et dtd pat og skriv NewsML objektet ut som en string
      * formatert som et XML dokument.
     */
    public String writeNewsML(String dtdDef){
        StringBuffer sb = new StringBuffer();
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + LF);
        if (dtdDef == null) {
            sb.append("<!DOCTYPE NewsML SYSTEM \"NewsML.dtd\">" + LF);
            sb.append("<?xml-stylesheet type=\"text/xsl\" href=\"NAAF.xsl\"?>" + LF);
        } else sb.append(dtdDef);
        sb.append("<NewsML>" + LF);
        sb.append(newsEnvelope.getNewsEnvelope());
        Iterator i = newsItems.iterator();
        while (i.hasNext()){
            sb.append(((NewsItem)i.next()).getNewsItem());
        }
        sb.append("</NewsML>" + LF);
        return sb.toString();
    }
    
    /**
     * Tar inn et filnavn og skriv dette NewsML objektet sine data til en
     * fil med det navn og path, formatert som et XML dokument.
     * Eksempel:
     *  writeNewsML("c:\\testNewsML.xml", null);
     *  writeNewsML("c:\\testNewsML.xml", "<!DOCTYPE NewsML SYSTEM \"http:\\www.bibsys.no\NewsML.dtd\">"");
     */
    public void writeNewsML(String filename, String dtdDef){
        try {
            File file = new File(filename);
            file.createNewFile(); // Oppretter ein ny fil, sletter ein evt. eksisterande.
            BufferedWriter bw = new BufferedWriter(new FileWriter(file));
            bw.write(writeNewsML(dtdDef));
            bw.flush(); // Tmmer bufferen ut p fila
            bw.close(); // Stenger fila
            System.out.println("Ferdig med filbehandling i Test, sjekk " + file.getPath());
        } catch (Exception e) {
            System.out.println("Noko gjekk galt i Test...");
            e.printStackTrace();
        }
    }
}
