package no.ingeberg.NewsML;
import java.net.*;
import java.util.*;

/**
 * NewsManagement.java <br>
 * Klassen hndterer innkomne data under "NewsManagement" elementet, <br>
 * som gir informasjon som er relevant for forvaltning av en nyhetspost, 
 * og informasjon viktigheten til informasjonen i nyhetsposten, om arv og assosiasjoner mellom ulike nyhetsposter, <br>
 * og/eller spesielle instruksjoner myntet p mottaker av informasjonen.
 * NewsManagement elementet er oblikatorisk og skal forekomme kun 1 gang.
 * Klassen innholder metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 * Disse metodene blir kalt i "NewsML" klassa som bygger setter sammen hele NewsItem elementet
 *
 * @author Marit Ingeberg
 */
public class NewsManagement {
    private String newsItemType         = null; //1 Verdiene hentes fra et kont. vokabular
    private URL newsItemTypeVocabulary  = null;
    private URL newsItemTypeScheme      = null;
    private String firstCreated         = null;//1
    private String thisRevisionCreated  = null; //1
    private String status               = null; //1 Verdiene hentes fra et kont. vokabular
    private URL statusVocabulary        = null;
    private URL statusScheme            = null;
    private String futureStatus         = null;//StatusWillChange forekommer 0-1 ganger, og bestr av futureStatus og dataAndTime
    private String dateAndTime          = null;
    private String urgency              = null; //0-1 Verdiene hentes fra et kont. vokabular
    private URL urgencyVocabulary       = null; //att. frivillig
    private URL urgencyScheme           = null; //att. frivillig
    private String revisionHistory      = null;//0-1
    //  private URL derivedFrom             = null;//0-N NB! URN
    private int derivedFromNr           = 0; //derivedFrom er en URL variabel som kan forekomme 0-N ganger
    private ArrayList derivedFrom       = null;
    private int associatedWithNr        = 0; //associatedWith er en URL variabel som kan forekomme 0-N ganger. NB! URN
    private ArrayList associatedWith    = null;
    private ArrayList instructionMap    = null; 
    //instruction er en string som kan forekomme 0-N ganger.Verdiene hentes fra et kont. vokabular
    private ArrayList propertyMap       =null; 
    //Elementet Property kan forekomme 0-Nganger, og bestr av property Formalname, og assignment, der assignment er frivillig. Kont. 
    //Verdiene hentes fra et kont. vokabular
    
    
    /**
     *Oppretter en ny instans av NewsManagement, i denne konstruktoren er alle under elementene tatt med.
     * @param newsItemType              Angir hvilke type en nyhetspost er av. Verdiene er hentet fra et kontrollert vokabular.<br><br>
     * @param newsItemTypeVocabulary    Angir kontrolert vokabular for verdiene. <br><br>
     * @param newsItemTypeScheme        Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     * @param firstCreated              Angir dato, og frivillig tiden som nyhetsposten frst ble laget. Bruker ISO 8601 Basic Format<br><br>
     * @param thisRevisionCreated       Angir revisjons dato for denne spesifikke nyhetsposten.<br><br>
     * @param status                    Angir nyhetspostens anvenlighet. Verdiene i dette elementet er hentet fra et kontrollert vokabular. 
     *                                  IPTC har definert et topicset som en kan bruke. Dette bestr flgende status verdier: 
     *                                  usable, - som forteller at nyhetsposten er uten restriksjoner. 
     *                                  Embargoed, - som er at nyhetsposten og dens innhold er stengt for offentliggjring inntil den 
     *                                  blir godkjent for dette av utgiver, Withheld, - nyhetsposten eller dens innhold er ikke klar for 
     *                                  utgivelse. canceled, - Verken nyhetsposten eller dens innhold skal under noen omstendigheter 
     *                                  offentliggjres.<br><br>
     * @param statusVocabulary          Angir kontrolert vokabular for verdiene. <br><br>
     * @param statusScheme              Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     * @param String futureStatus       Status elementet angir nr nyhetsposten automatisk kommer til  bli endret. 
     *                                  Her angir en underelementene, FutureStatus, som angir ny status og DateAndTime, som angir dato for 
     *                                  endringen. Elementet er frivillig.<br><br>
     * @param dateAndTime               Status elementet angir nr nyhetsposten automatisk kommer til  bli endret. 
     *                                  Her angir en under elementene, FutureStatus, som angir ny status og DateAndTime, som angir dato 
     *                                  for endringen. Elementet er frivillig.<br><br>
     * @param urgency                   Angir hvor viktig nyhetsposten anses for  vre. Verdiene hentes fra et kontrollert vokabular.<br><br>
     * @param urgencyVocabulary         Angir kontrolert vokabular for verdiene. <br><br>
     * @param urgencyScheme             Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     * @param revisionHistory           Angir en peker til en fil som innholder revisjons historien til nyhetsposten, angis ved et 
     *                                  Href attributt. Utgiver kan selv velge syntaks og struktur p denne revisjonshistorie fila. 
     *                                  Frivillig element.<br><br>
     * @param derivedFrom               Angir en peker til den nyhetsposten den aktuelle er derivert fra. Dette gjres ved et 
     *                                  NewsItem attributt, som kan vre en http URL eller en NewsML URN. Frivillig og repeterbart.<br><br>
     * @param associatedWith            Angir en peker til den nyhetsposten den aktuelle er assosiert med. Dette kan for eksempel vre bilder, 
     *                                  eller andre artikler om samme tema. Dette gjres ved et NewsItem attributt, som kan vre en http URL 
     *                                  eller en NewsML URN. Har ogs et frivillig kommentar felt. Frivillig og repeterbart.<br><br>
     * @param instruction               Inneholder instrukser fra nyhets utgiver til det som mottar nyhetsposten. Disse instruksjonene hentets 
     *                                  fra et kontrolert vokabular. Frivillig og repeterbart.<br><br>
     * @param instructionVocabulary     Angir kontrolert vokabular for verdiene i "instruction". <br><br>
     * @param propertyFormalname        Angir navn for "tilleggsegenskaper" ved contentItem, topic, NewsComponent og newsItem.<br><br>
     * @param propertyAssignment        Angir verdi for "tilleggsegenskaper" ved contentItem, topic, NewsComponent og newsItem.<br><br>
     */
    public NewsManagement(String newsItemType,  URL newsItemTypeVocabulary,
    URL newsItemTypeScheme, String firstCreated ,String thisRevisionCreated,
    String status,URL statusVocabulary, URL statusScheme, String futureStatus,
    String dateAndTime, String urgency, URL urgencyVocabulary,
    URL urgencyScheme, String revisionHistory, URL derivedFrom,
    URL associatedWith, String instruction, URL instructionVocabulary, String propertyFormalname, String propertyAssignment){
        this.newsItemType           = newsItemType;
        this.newsItemTypeVocabulary = newsItemTypeVocabulary;
        this.newsItemTypeScheme     = newsItemTypeScheme;
        this.firstCreated           = firstCreated;
        this.thisRevisionCreated    = thisRevisionCreated;
        this.status                 = status;
        this.statusVocabulary       = statusVocabulary;
        this.statusScheme           = statusScheme;
        this.futureStatus           = futureStatus;    //StatusWillChange
        this.dateAndTime            = dateAndTime;    //StatusWillChange //getTimeAndDate();
        this.urgency                = urgency;
        this.urgencyVocabulary      = urgencyVocabulary;
        this.urgencyScheme          = urgencyScheme;
        this.revisionHistory        = revisionHistory;
        this.derivedFrom            = new ArrayList();
        this.derivedFrom.add(derivedFrom);
        this.associatedWith         = new ArrayList();
        this.associatedWith.add(associatedWith);
        this.instructionMap         = new ArrayList();
        addInstructionMap(instruction,instructionVocabulary);
        this.propertyMap            = new ArrayList();
        addPropertyMap(propertyFormalname, propertyAssignment);
    }
    
    /**
     *Oppretter en ny instans av NewsManagement, i denne konstruktoren er alle under elementene tatt med.
     * @param newsItemType              Angir hvilke type en nyhetspost er av. Verdiene er hentet fra et kontrollert vokabular.<br><br>
     * @param firstCreated              Angir dato, og frivillig tiden som nyhetsposten frst ble laget. Bruker ISO 8601 Basic Format<br><br>
     * @param thisRevisionCreated       Angir revisjons dato for denne spesifikke nyhetsposten.<br><br>
     * @param status                    Angir nyhetspostens anvenlighet. Verdiene i dette elementet er hentet fra et kontrollert vokabular. 
     *                                  IPTC har definert et topicset som en kan bruke. Dette bestr flgende status verdier: usable, 
     *                                  - som forteller at nyhetsposten er uten restriksjoner. Embargoed, - som er at nyhetsposten og dens 
     *                                  innhold er stengt for offentliggjring inntil den blir godkjent for dette av utgiver, 
     *                                  Withheld, - nyhetsposten eller dens innhold er ikke klar for utgivelse. canceled, 
     *                                  - Verken nyhetsposten eller dens innhold skal under noen omstendigheter offentliggjres.<br><br>
     * @param statusScheme              Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     * @param String futureStatus       Status elementet angir nr nyhetsposten automatisk kommer til  bli endret. Her angir en 
     *                                  underelementene, FutureStatus, som angir ny status og DateAndTime, som angir dato for endringen.
     *                                  Elementet er frivillig.<br><br>
     * @param urgency                   Angir hvor viktig nyhetsposten anses for  vre. Verdiene hentes fra et kontrollert vokabular.<br><br>
     */
    public NewsManagement(String newsItemType, String firstCreated,String thisRevisionCreated,
    String status, String futureStatus,String dateAndTime, String urgency){
        this.newsItemType           = newsItemType;
        this.firstCreated           = firstCreated;
        this.thisRevisionCreated    = thisRevisionCreated;
        this.status                 = status;
        this.futureStatus           = futureStatus;    //StatusWillChange
        this.dateAndTime            = dateAndTime;    //StatusWillChange //getTimeAndDate();
        this.urgency                = urgency;
    }
    
    
    
    /**
     *
     * Minste lovlige konstruktoren, alle NewsManagement m innholde minst flgende element:
     * "NewsItemType", "FirstCreated", "ThisRevisionCreated", og "Status", med tilhrende
     * underverdier.
     * @param newsItemType              Angir hvilke type en nyhetspost er av. Verdiene er hentet fra et kontrollert vokabular.<br><br>
     * @param firstCreated              Angir dato, og frivillig tiden som nyhetsposten frst ble laget. Bruker ISO 8601 Basic Format<br><br>
     * @param thisRevisionCreated       Angir revisjons dato for denne spesifikke nyhetsposten.<br><br>
     * @param status                    Angir nyhetspostens anvenlighet. Verdiene i dette elementet er hentet fra et kontrollert vokabular. 
     *                                  IPTC har definert et topicset som en kan bruke. Dette bestr flgende status verdier: 
     *                                  usable, - som forteller at nyhetsposten er uten restriksjoner. Embargoed, - som er at nyhetsposten 
     *                                  og dens innhold er stengt for offentliggjring inntil den blir godkjent for dette av utgiver, 
     *                                  Withheld, - nyhetsposten eller dens innhold er ikke klar for utgivelse. canceled, 
     *                                  - Verken nyhetsposten eller dens innhold skal under noen omstendigheter offentliggjres.<br><br>
     */
    public NewsManagement(String newsItemType, String firstCreated, String thisRevisionCreated, String status){
        this.newsItemType           = newsItemType;
        this.firstCreated           = firstCreated;
        this.status                 = status;
        this.thisRevisionCreated    = thisRevisionCreated; //getTimeAndDate();
    }
    
    /**
     * Metoden setter newsItemTypeVocabulary
     * @param newsItemTypeVocabulary    Angir kontrolert vokabular for verdiene. <br><br>
     */
    public void setNewsItemTypeVocabulary(URL newsItemTypeVocabulary){
        this.newsItemTypeVocabulary = newsItemTypeVocabulary;
    }
    
    /**
     *Metoden setter newsItemTypeScheme
     * @param newsItemTypeScheme        Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     */
    public void setNewsItemTypeScheme(URL newsItemTypeScheme){
        this.newsItemTypeScheme     = newsItemTypeScheme;
    }
    /**
     * Metoden setter statusVocabulary
     *  * @param statusVocabulary          Angir kontrolert vokabular for verdiene. <br><br>
     */
    public void setStatusVocabulary(URL statusVocabulary){
        this.statusVocabulary       = statusVocabulary;
    }
    
    /**
     * Metoden setter statusScheme
     * @param statusScheme Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     */
    public void setStatusScheme(URL statusScheme){
        this.statusScheme           = statusScheme;
    }
    
    /**
     * Metoden setter StatusWillChange
     * @param String futureStatus       Status elementet angir nr nyhetsposten automatisk kommer til  bli endret. 
     *                                  Her angir en under elementene, FutureStatus, som angir ny status og DateAndTime, som angir dato 
     *                                  for endringen. Elementet er frivillig.<br><br>
     * @param dateAndTime               Status elementet angir nr nyhetsposten automatisk kommer til  bli endret. Her angir en under 
     *                                  elementene, FutureStatus, som angir ny status og DateAndTime, som angir dato for endringen. 
     *                                  Elementet er frivillig.<br><br>
     */
    public void setStatusWillChange(String futureStatus, String dateAndTime){
        this.futureStatus           = futureStatus;
        this.dateAndTime            = dateAndTime;
    }
    
    /**
     * Metoden setter urgency
     * @param urgency                   Angir hvor viktig nyhetsposten anses for  vre. Verdiene hentes fra et kontrollert vokabular.<br><br>
     * @param urgencyVocabulary         Angir kontrolert vokabular for verdiene. <br><br>
     * @param urgencyScheme             Angir skjema for det gir mening til verdiene i det kontrollerte vokabularet.<br><br>
     *
     */
    public void setUrgency(String urgency, URL urgencyVocabulary, URL urgencyScheme){
        this.urgency                = urgency;
        this.urgencyVocabulary      = urgencyVocabulary;
        this.urgencyScheme          = urgencyScheme;
    }
    
    /**
     * Metoden setter revisionHistory
     * @param revisionHistory  Angir en peker til en fil som innholder revisjons historien til nyhetsposten, angis ved et Href attributt.
     *                         Utgiver kan selv velge syntaks og struktur p denne revisjonshistorie fila. Frivillig element.<br><br>
     */
    public void setRevHistory(String revisionHistory){
        this.revisionHistory        = revisionHistory;
    }
    
    /**Metoden legger inn en ekstra derivedFrom element
     * @param derivedFrom               Angir en peker til den nyhetsposten den aktuelle er derivert fra. Dette gjres ved et 
     *                                  NewsItem attributt, som kan vre en http URL eller en NewsML URN. Frivillig og repeterbart.<br><br>
     */
    public void addDerivedFrom(URL derivedFrom){
        this.derivedFrom.add(derivedFrom);
    }
    
    /**Metoden legger inn en ekstra associatedWith element
     * @param associatedWith            Angir en peker til den nyhetsposten den aktuelle er assosiert med. 
     *                                  Dette kan for eksempel vre bilder, eller andre artikler om samme tema. Dette gjres ved 
     *                                  et NewsItem attributt, som kan vre en http URL eller en NewsML URN. Har ogs et frivillig
     *                                  kommentar felt. Frivillig og repeterbart.<br><br>
     */
    public void addAssociatedWith(URL associatedWith){
        this.associatedWith.add(associatedWith);
    }
    
    /**Metoden legger inn en ekstra instructon element, med bde String instruction og URL instructionVocabulary verdier
     * @param instruction               Inneholder instrukser fra nyhets utgiver til det som mottar nyhetsposten. Disse 
     *                                  instruksjonene hentets fra et kontrolert vokabular. Frivillig og repeterbart.<br><br>
     * @param instructionVocabulary     Angir kontrolert vokabular for verdiene i "instruction". <br><br>
     */
    public void addInstructionMap(String instruction, URL instructionVocabulary){
        this.instructionMap.add(new InstructionMap(instruction,instructionVocabulary));
    }
    
    /**Metoden legger inn en ekstra instructon element, med "bare" String instruction som parametere
     * @param instruction               Inneholder instrukser fra nyhets utgiver til det som mottar nyhetsposten. 
     *                                  Disse instruksjonene hentets fra et kontrolert vokabular. Frivillig og repeterbart.<br><br>
     */
    public void addInstructionMap( String instruction){
        this.instructionMap.add(new InstructionMap(instruction));
    }
    
    /**Metoden legger inn en ekstra property element, med bde propertyFormalname og propertyAssignment verdier
     * @param propertyFormalname        Angir navn for "tilleggsegenskaper" ved contentItem, topic, NewsComponent og newsItem.<br><br>
     * @param propertyAssignment        Angir verdi for "tilleggsegenskaper" ved contentItem, topic, NewsComponent og newsItem.<br><br>
     */
    public void addPropertyMap(String propertyFormalname, String propertyAssignment){
        this.propertyMap.add(new PropertyMap(propertyFormalname, propertyAssignment));
    }
    
    /**Metoden legger inn en ekstra property element, med propertyFormalname verdi
     * @param propertyFormalname        Angir navn for "tilleggsegenskaper" ved contentItem, topic, NewsComponent og newsItem.<br><br>
     */
    public void addPropertyMap(String propertyFormalname){
        this.propertyMap.add(new PropertyMap(propertyFormalname));
    }
    
    /**
     * Metoden returnerer alle NewsManagement variablene, i xml-format.
     *
     */
    public String getNewsManagement() {
        /**  <NewsManagement>
         * <NewsItemType FormalName="News" Vocabulary="urn:newsml:naaf.no:20021120:IptcNewsItemTypes:1" Scheme="NaafNewsItemTypes"/>
         * <FirstCreated>20021120T+0100</FirstCreated>
         * <ThisRevisionCreated>20021120T+0100</ThisRevisionCreated>
         * <Status Vocabulary="urn:newsml:naaf.no:20021120:NaafStatus:1" Scheme="NaafStatus" FormalName="Embargoed"/>
         * <StatusWillChange>
         * <FutureStatus FormalName="Usable"/>
         * <DateAndTime>20021120T1200+0000</DateAndTime>
         * </StatusWillChange>
         * <Urgency Vocabulary="urn:newsml:naaf.no:20021120:NaafUrgency:1" Scheme="NaafUrgency" FormalName="1"/>
         * <RevisionHistory Href="../history/rev_1376.log"/>
         * <DerivedFrom NewsItem="urn:newsml:Naaf.no:20021120:NewsML%201.0%20approved"/>
         * <AssociatedWith NewsItem="urn:newsml:naaf.no:20021120:NewsML%201.0%20approved"/>
         * <Instruction FormalName="CancelAll" Vocabulary="#MyInstructionCodes"/>
         * <Property FormalName="Width">
         * <Property FormalName="Quantity" Value="7.5"/>
         * <Property FormalName="Unit" ValueRef="urn:newsml:mydomain.com:20021120:Units:1#cm"/>
         * </Property>
         * </NewsManagement>
         */
        String t = "<NewsManagement>";
        t = t + "<NewsItemType FormalName=\"" + newsItemType + "\" ";
        if (newsItemTypeVocabulary != null)t = t +"\" Vocabulary=\"" + newsItemTypeVocabulary + "\" ";
        if (newsItemTypeScheme !=null) t =t + "Scheme=\"" + newsItemTypeScheme + "\"" ;
        t = t + "/> ";
        t = t + "<FirstCreated> " + firstCreated + "</FirstCreated>";
        t = t + "<ThisRevisionCreated>" + thisRevisionCreated +"</ThisRevisionCreated>";
        
        t = t+ "<Status FormalName=\"" + status + "\" ";
        if (statusVocabulary != null) t = t+ "Vocabulary=\"" + statusVocabulary + "\" ";
        if (statusScheme != null) t = t + "Scheme=\"" + statusScheme + "\"";
        t = t + "/> ";
        
        if (futureStatus != null){
            t = t + "<StatusWillChange>";
            t = t + "<FutureStatus FormalName=\"" + futureStatus + "\"/>";
            t = t + "<DateAndTime>" + dateAndTime + "</DateAndTime>";
            t = t + "</StatusWillChange>";
        }
        
        if (urgency != null){
            t = t + "<Urgency FormalName=\"" + urgency + "\" ";
            if (urgencyVocabulary != null) t = t + "Vocabulary=\"" + urgencyVocabulary + "\" ";
            if (urgencyScheme !=null) t = t + "Scheme=\"" +urgencyScheme +"\" ";
            t = t + "/> ";
        }
        
        if (revisionHistory !=null) t = t + "<RevisionHistory Href=\"" + revisionHistory + "\"/>";
        if (derivedFrom !=null){
            Iterator i = derivedFrom.iterator();
            while (i.hasNext()) t = t + "<DerivedFrom NewsItem=\"" + i.next().toString() + "\"/>";
        }
        if (associatedWith != null){
            Iterator i = associatedWith.iterator();
            while (i.hasNext()) t = t + "<AssociatedWith NewsItem=\"" + i.next().toString() + "\"/>";
        }
        if (instructionMap != null){
            Iterator i = instructionMap.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        if (propertyMap != null){
            Iterator i = propertyMap.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        t = t + "</NewsManagement>";
        return t;
    }
    
    ////////////////////////////////////////////////
    ////////Subclass Instruction///////////////////
    //////////////////////////////////////////////
    
    private class InstructionMap{
        private String instruction        = null;
        private URL instructionVocabulary = null;
        
        
        public InstructionMap(String instruction, URL instructionVocabulary){
            this.instruction = instruction;
            this.instructionVocabulary = instructionVocabulary;
        }
        
        public InstructionMap(String instruction){
            this.instruction = instruction;
        }
        
        public String toString(){
            if (instructionVocabulary != null)
                return new String("<Instruction FormalName=\"" + instruction + "\" Vocabulary=\"" + instructionVocabulary + "\"/>");
            return new String("<Instruction FormalName=\"" + instruction + "\"/>");
        }
    }
    
    ////////////////////////////////////////////////
    ////////Subclass PropertyMap//////////////////
    //////////////////////////////////////////////
    
    private class PropertyMap{
        private String propertyFormalname = null;
        private String propertyAssignment = null;
        
        public PropertyMap(String propertyFormalname, String propertyAssignment){
            this.propertyFormalname = propertyFormalname;
            this.propertyAssignment = propertyAssignment;
        }
        
        public PropertyMap(String propertyFormalname){
            this.propertyFormalname = propertyFormalname;
        }
        
        public String toString(){
            if (propertyAssignment != null)
                return new String("<Property FormalName=\"" + propertyFormalname + "\" ValueRef=\"" + propertyAssignment + "\"/>");
            return new String("<Property FormalName=\"" + propertyFormalname + "\"/>");
        }
    }
}