package no.ingeberg.NewsML;
import java.net.*;
import java.util.*;
/**
 * Resource..java <br>
 * Klassen hndterer innkomne data under "Resource" elementet, som indikerer hvor en gitt ressurs kan bli funnet,
 * og om den kan bli brukt som default vokabular for visse formelle navn innen det aktueller subtreet av NewsML dokumentet.
 * Klassen innholder metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */
public class Resource {
    private URL urn             = null;
    private ArrayList url       = null;
    private ArrayList defVocMap = null;
    
    /**
     *Oppretter en ny istans av catalog, i denne constructoren er alle under elementene tatt med.
     * @param urn           Identifiserer den eksterne ressursen.
     * @param url           Identifiserer den eksterne ressursen.
     * @param defVocContext Innholder et XPath mnster. Den identifiserte ressursen fungerer som standard vokabular for alle elementene og 
     *                      attributtene som stemmer overens XPath mnsteret. Om  XPath mnsteret stemmer med et element, vil  verdien for 
     *                      FormalName attributtet til elementet angitt. Stemmer XPath mnsteret med et attributt, vil verdien til dette 
     *                      attributtet selv, bli angitt. 
     * @param defVocScheme  Angir skjema for default vokabularet.
     */
    public Resource(URL urn, URL url, String defVocContext, String defVocScheme){
        this.urn             = urn;
        this.url             = new ArrayList();
        this.url.add(url);
        this.defVocMap       = new ArrayList();
        addDefVocMap(defVocContext, defVocScheme);
    }
    
    /** Oppretter et tomt objekt. */
    public Resource(){
        //det er lov  sende inn en tom konstruktor
    }
    
    /** Metoden setter URN element 
     * @param urn           Identifiserer den eksterne ressursen.
     */
    public void setUrn(URL urn){
        this.urn = urn;
    }
    
    /** Metoden legger til et "ekstra" URL element Kan gjentaes.
     * @param url           Identifiserer den eksterne ressursen.
     */
    public void addUrl(URL url){
        this.url.add(url);
    }
    
    /** Metoden legger til et "ekstra" DefaultVocabulary element som bestr av elementene defVocContext og defVocSchemeelement. Kan gjentaes.
     * @param defVocContext Innholder et XPath mnster. Den identifiserte ressursen fungerer som standard vokabular for alle elementene og 
     *                      attributtene som stemmer overens XPath mnsteret. Om  XPath mnsteret stemmer med et element, vil  verdien for  
     *                      FormalName attributtet til elementet angitt. Stemmer XPath mnsteret med et attributt, vil verdien til dette 
     *                      attributtet selv, bli angitt. 
     * @param defVocScheme  Angir skjema for default vokabularet.
     */
    public void addDefVocMap(String defVocContext, String defVocScheme){
        this.defVocMap.add(new DefVocMap(defVocContext, defVocScheme));
    }
    
    /** Metoden legger til et "ekstra" DefaultVocabulary element med bare elementet defVocContext som parameter. Kan gjentaes.
     * @param defVocContext Innholder et XPath mnster. Den identifiserte ressursen fungerer som standard vokabular for alle elementene og 
     *                      attributtene som stemmer overens XPath mnsteret. Om  XPath mnsteret stemmer med et element, vil  verdien for 
     *                      FormalName attributtet til elementet angitt. Stemmer XPath mnsteret med et attributt, vil verdien til dette 
     *                      attributtet selv, bli angitt. 
     */
    public void addDefVocMap(String defVocContext){
        this.defVocMap.add(new DefVocMap(defVocContext));
    }
    
    /**
     * Metoden returnerer alle resouce variablene, i xml-format.
     *
     */
    public String getResource(){
        String t = "<Resource>";
        if (urn != null) t = t + "<Urn>" + urn + "</Urn>";
        if (url != null){
            Iterator i = url.iterator();
            while (i.hasNext()) t = t + "<Url>" + i.next().toString() + "</Url>";
        }
        if (defVocMap != null){
            Iterator i = defVocMap.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        t = t + "</Resource>";
        return t;
    }
    
    
    ////////////////////////////////////////////////
    ////////Subclass DefVocMap///////////////////
    //////////////////////////////////////////////
    private class DefVocMap{
        private String defVocContext =null;
        private String defVocScheme = null;
        
        public DefVocMap(String defVocContext, String defVocScheme){
            this.defVocContext = defVocContext;
            this.defVocScheme  = defVocScheme;
        }
        
        public DefVocMap(String defVocContext){
            this.defVocContext = defVocContext;
        }
        
        public String toString(){
            if (defVocScheme != null)
                return new String("<DefaultVocabularyFor Context=\"" + defVocContext + "\" Scheme=\"" + defVocScheme + "\"/>");
            return new String( "<DefaultVocabularyFor Context=\"" + defVocContext +"\"/>");
        }
    }
}