package no.ingeberg.NewsML;
import java.net.*;
import java.io.*;
import java.util.*;
/**
 * RightsMetadata.java <br>
 * Klassen hndterer innkomne data under "RightsMetadata" elementet,
 * som ligger under NewsComponent elementet. Dette elementet er frivillig og kan
 * kun forekomme 1 gang. Klassen kaller ved behov "Catalog" klassen som tar seg
 * av "Catalog elementetene som kan vre endel av denne klassen.
 * Klassen innholder metoder for  hente ut variablene som blir satt for dette
 * elementet, formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class RightsMetadata {
    private Catalog catalog         = null;
    private ArrayList copyright     = null;
    private ArrayList usageRights   = null;
    private ArrayList property      = null; //for ekstra matadata, ikke inkludert i dtd. Kan forekomme 0-N ganger
    
    /**
     *Oppretter en ny instans av catalog, i denne konstruktoren er alle under
     *elementene tatt med. Alle elementene her er egentlig frivillige.
     * @param catalog           identifiserer standard vokabular for DescriptionMetadata. <br>
     *                          Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                          Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param copyrightComment 	Copyright, innholder underelementene comment, CopyrightHolder og CopyrightDate.  
     *                          Her har en et kommentar element, to element som i naturlig sprk gir informasjon om hvem 
     *                          copyrighten tilhrer og angir dato for den.<br><br>
     * @param copyrightHolder   Angir i naturlig sprk informasjon om hvem copyright'en tilhrer.<br><br>
     * @param copyrightDate     Angir i naturlig sprk informasjon om  dato for copyright'en.<br><br>
     * @param usageType         Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til 
     *                          nyhetskomponenten. usageType gir informasjon i naturlig sprk om hvilke type bruk 
     *                          rettighetene retter seg mot.<br><br>
     * @param usageGeography    Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til 
     *                          nyhetskomponenten. Geography, gir informasjon om hvilke geografiske omrder rettighetene 
     *                          gjelder for.<br><br>
     * @param usageRightsHolder Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til 
     *                          nyhetskomponenten. RightHolder, forteller hvem som rettighetene tilhrer.<br><br>
     * @param usageLimitations  Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          "Limitations" angir eventuelle restriksjoner p bruken av innholdet i nyhetskomponenten.<br><br>
     * @param usageStartDate    Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          Angir tidsrommet rettighetene for denne nyhetsposten varer.<br><br>
     * @param usageEndDate      Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          Angir tidsrommet rettighetene for denne nyhetsposten varer.<br><br>
     * @param property          Brukes til  beskrive vrig rettighets metadata som ikke blir dekket av de vrige. Frivillig og repeterbart
     */
    public RightsMetadata(Catalog catalog, ArrayList copyrightComment,
    String copyrightHolder, String copyrightDate, String usageType, String usageGeography,
    String usageRightsHolder, String usageLimitations, String usageStartDate,
    String usageEndDate, String property){
        setCatalog(catalog);
        addProperty(property);
        addCopyright(copyrightComment, copyrightHolder, copyrightDate);
        addUsageRights(usageType, usageGeography, usageRightsHolder, usageLimitations, usageStartDate, usageEndDate);
    }
    
    /** Oppretter et tomt ReghtsMetadata objekt*/
    public RightsMetadata(){
        //Det er tillatt  opprette et tomt objekt.
    }
    
    /** Setter catalog elementet 
     * @param catalog   identifiserer standard vokabular for DescriptionMetadata. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     */
    public void setCatalog(Catalog catalog){
        this.catalog = catalog;
    }
    /* Legger til et copyright element
     * @param copyrightComment 	Copyright, innholder underelementene comment, CopyrightHolder og CopyrightDate.  Her har en et kommentar 
                                element, to element som i naturlig sprk gir informasjon om hvem copyrighten tilhrer og 
                                angir dato for den.<br><br>
     * @param copyrightHolder   Angir i naturlig sprk informasjon om hvem copyright'en tilhrer.<br><br>
     * @param copyrightDate     Angir i naturlig sprk informasjon om  dato for copyright'en.<br><br>
     */
    public void addCopyright(ArrayList copyrightComment, String copyrightHolder, String copyrightDate){
        if (this.copyright == null) this.copyright = new ArrayList();
        this.copyright.add(new RightsMetadata.Copyright(copyrightComment, copyrightHolder, copyrightDate));
    }
    
    /* Legger til et copyright element
     * @param copyrightHolder   Angir i naturlig sprk informasjon om hvem copyright'en tilhrer.<br><br>
     * @param copyrightDate     Angir i naturlig sprk informasjon om  dato for copyright'en.<br><br>
     */
    public void addCopyright(String copyrightHolder, String copyrightDate){
        if (this.copyright == null) this.copyright = new ArrayList();
        this.copyright.add(new RightsMetadata.Copyright(copyrightHolder, copyrightDate));
    }
    
    /**
     *Legger til et usageRights element
     * @param usageType         Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          usageType gir informasjon i naturlig sprk om hvilke type bruk rettighetene retter seg mot.<br><br>
     * @param usageGeography    Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          Geography, gir informasjon om hvilke geografiske omrder rettighetene gjelder for.<br><br>
     * @param usageRightsHolder Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          RightHolder, forteller hvem som rettighetene tilhrer.<br><br>
     * @param usageLimitations  Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          "Limitations" angir eventuelle restriksjoner p bruken av innholdet i nyhetskomponenten.<br><br>
     * @param usageStartDate    Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          Angir tidsrommet rettighetene for denne nyhetsposten varer.<br><br>
     * @param usageEndDate      Del av UsageRights, som angir informasjon om hvem som innehar rettighetene til nyhetskomponenten.
     *                          Angir tidsrommet rettighetene for denne nyhetsposten varer.<br><br>
     */
    public void addUsageRights(String usageType, String usageGeography,
    String usageRightsHolder, String usageLimitations, String usageStartDate,
    String usageEndDate){
        if (this.usageRights == null) this.usageRights = new ArrayList();
        this.usageRights.add(new RightsMetadata.UsageRights(usageType, usageGeography,
        usageRightsHolder, usageLimitations, usageStartDate, usageEndDate));
    }
    
    /** Legger til et property element 
     * @param property  Brukes til  beskrive vrig rettighets metadata som ikke blir dekket av de vrige. Frivillig og repeterbart     
     */
    public void addProperty(String property){
        if (this.property == null) this.property = new ArrayList();
        this.property.add(property);
    }
    
    /**
     * Metoden returnerer alle rightsMetadata variablene, i xml-format.
     *
     */
    public String getRightsMetadata(){
            /*
             <RightsMetadata>
                <Catalog Href="som beskrevet over."/>
                <Copyright>
                        <CopyrightHolder>Kommentar felt som i narturlig sprk angir informasjon om hvem copyrighten tilhrer </CopyrightHolder>
                        <CopyrightDate>Kommentar felt som i narturlig sprk angir dato for copyright</CopyrightDate>
                </Copyright>
                <UsageRights>
                        <UsageType>gir informasjon i naturlig sprk om hvilke type bruk rettighetene retter seg mot.</UsageType>
                        <Geography>gir informasjon om hvilke geografiske omrder rettighetene gjelder for.</Geography>
                        <RightsHolder>forteller hvem som rettighetene tilhrer</RightsHolder>
                        <StartDate>Angir tidsrommet rettighetene for denne nyhetsposten varer.</StartDate>
                        <EndDate>Angir tidsrommet rettighetene for denne nyhetsposten varer.</EndDate>
                        </UsageRights>
                </RightsMetadata>
             *            **/
        
        String t ="<RightsMetadata>";
        if (catalog !=null){t= t + catalog.getCatalog();}
        if (copyright != null){
            Iterator i = copyright.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        if (usageRights != null) {
            Iterator i = usageRights.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        if (property != null){
            Iterator i = property.iterator();
            while (i.hasNext()) t = t + "<Property FormalName=\"" + i.next().toString() + "\"/>";
        }
        t= t + "</RightsMetadata>";
        return t;
    }
    
    
    /////////////////////////////////////////////////
    //////////////// PRIVATE CLASS //////////////////
    /////////////////////////////////////////////////
    private class Copyright {
        ArrayList copyrightComment  = null;
        String copyrightHolder      = null;
        String copyrightDate        = null;
        
        public Copyright(ArrayList copyrightComment, String copyrightHolder, String copyrightDate) {
            this.copyrightComment   = new ArrayList(copyrightComment);
            this.copyrightHolder    = copyrightHolder;
            this.copyrightDate      = copyrightDate;
        }
        
        public Copyright(String copyrightHolder, String copyrightDate) {
            this.copyrightHolder    = copyrightHolder;
            this.copyrightDate      = copyrightDate;
        }
        
        public String toString() {
            String t = new String("<Copyright>");
            if (copyrightComment != null){
                Iterator i = copyrightComment.iterator();
                while (i.hasNext()) t = t + "<Comment>" + i.next().toString() + "</Comment>";
            }
            t = t + "<CopyrightHolder>" + copyrightHolder + "</CopyrightHolder> ";
            t = t + "<CopyrightDate>" + copyrightDate + "</CopyrightDate>";
            t= t + "</Copyright>";
            return t;
        }
        
        /* Denne metoden kan brukes til  legge til flere copyright-comment, til den enkelte copyright objekt.
        public void setUsageComment(ArrayList copyrightComment){
            if (this.copyrightComment ==null) this.copyrightComment = new ArrayList();
            this.copyrightComment.add(copyrightComment);
        }
         */
    }
    
    /////////////////////////////////////////////////
    ///////// PRIVATE CLASS UsageRights /////////////
    /////////////////////////////////////////////////
    
    private class UsageRights {
        private String usageType        = null;
        private String usageGeography   = null;
        private String usageRightsHolder= null;
        private String usageLimitations = null;
        private String usageStartDate   = null;
        private String usageEndDate     = null;
        
        /**
         *      Konstruktor'n lager et objekt som innholder UsageRights elementet.
         *      Objektet vil bli persistent, dvs objektet kan ikke endres etter
         *      instansiering. Ingen av variablene i konstruktoren er obligatoriske,
         *      s metoden kan kalles med noen eller alle stringene som null-formaterte.
         *       opprette et objekt med KUN null-formaterte stringer vil medfre at
         *      toString metoden returner en tom string, - instansieringer
         *      tar da bare opp cpu tid.
         */
        public UsageRights(String usageType, String usageGeography,
        String usageRightsHolder, String usageLimitations, String usageStartDate,
        String usageEndDate) {
            this.usageType = usageType;
            this.usageGeography = usageGeography;
            this.usageRightsHolder = usageRightsHolder;
            this.usageLimitations = usageLimitations;
            this.usageStartDate = usageStartDate;
            this.usageEndDate = usageEndDate;
        }
        
        /**
         * Metoden returnerer UsageRights array'et som en ferdig xml formatert string,
         *  som innholder alle UsageRights elementene.
         */
        public String toString() {
            if ( !(usageType !=null) && (usageGeography !=null) && (usageRightsHolder !=null) && ( usageStartDate!=null) && 
                ( usageEndDate !=null)){
                //Hvis alle verdinen lik null
                return new String("");
            }
            String t = new String("<UsageRights>");
            if (usageType !=null){t= t + "<UsageType>" + usageType + "</UsageType>";}
            if (usageGeography !=null){t= t + "<Geography>" + usageGeography + "</Geography>";}
            if (usageRightsHolder !=null){t= t + "<RightsHolder>" + usageRightsHolder + "</RightsHolder>";}
            if ( usageStartDate!=null){t= t + "<StartDate>" + usageStartDate + "</StartDate>";}
            if ( usageEndDate !=null){t= t + "<EndDate>" + usageEndDate + "</EndDate>";}
            t= t + "</UsageRights>";
            return t;
        }
        
/*      Disse metodene m videre utvikles, for  sttte endring av eksisterende objekt.
        public void addUsageType(String usageType){
            this.usageType = usageType;
        }
 
        public void addUsageGeography(String usageGeography){
            this.usageGeography = usageGeography;
        }
 
        public void addUsageRightsHolder(String usageRightsHolder){
            this.usageRightsHolder = usageRightsHolder;
        }
 
        public void addUsageLimitations(String usageLimitations){
            this.usageLimitations = usageLimitations;
 
        }
 
        public void addUsageStartDate(String usageStartDate){
            this.usageStartDate = usageStartDate;
        }
 
        public void addUsageEndDate(String usageEndDate){
            this.usageEndDate = usageEndDate;
 
        }
 */
        
    }
}