package no.ingeberg.NewsML;
import java.net.*;
import java.util.*;
/**
 * Topic.java <br>
 * Klassen hndterer innkomne data under "topic" elementet, som er endel av TopicSet elementet som beskriver emnet 
 * til deler eller hele nyhetsposter. Klassen innholder metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */
public class Topic {
    private URL topicDuid               = null; //att
    private int commentNr               = 0; //Comment er en string som kan ha 0-N forekomster
    private ArrayList comment           = null;
    private Catalog catalog             = null; //0-1
    private ArrayList topicType         = null; //0-N
    private ArrayList descriptionMap    = null; //0-N
    private int propertyNr              = 0;//property er en string som kan ha 0-N forekomster
    private ArrayList property          = null;
    
    
    /**
     *Oppretter en ny instans av Topic, i denne konstruktoren er alle under
     * elementene tatt med.
     * @param topicDuid             Attributt som identifiserer topic'et unikt.<br><br>
     * @param comment               Legger til tilleggs informasjon i naturlig sprk.<br><br>
     * @param catalog               Identifiserer standard vokabular for Topic'et. <br>
     *                              Den kan og indikerer hvor i NewsML dokumentet Topicet er brukt. <br>
     *                              Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param topicType             Forteller av hvilken typet emnet er av.<br><br>
     * @param description           Beskriver emnet.<br><br>
     * @param descriptionXmlLang    Angir sprket beskrivelsen er gitt i.<br><br>
     * @param property              Angir verdien for spesifikke egenskaper ved emnet, som ikke er dekket av de vrige. <br><br>
     *
     */
    public Topic(URL topicDuid, String comment, Catalog catalog,
    String topicType, String description, String descriptionXmlLang, String property) {
        this.topicDuid          = topicDuid;
        this.comment             = new ArrayList();
        this.catalog             = catalog;
        this.topicType           = new ArrayList();
        this.descriptionMap      = new ArrayList();
        this.property            = new ArrayList();
        this.comment.add(comment);
        this.topicType.add(topicType);
        this.property.add(property);
    }
    
     /**
     *Oppretter en ny instans av Topic
     * @param comment               Legger til tilleggs informasjon i naturlig sprk.<br><br>
     * @param topicType             Forteller av hvilken typet emnet er av.<br><br>
     * @param description           Beskriver emnet.<br><br>
     */
    public Topic(String comment, String topicType, String description, String descriptionXmlLang) {
        this.comment             = new ArrayList();
        this.topicType           = new ArrayList();
        this.descriptionMap      = new ArrayList();
        this.comment.add(comment);
        this.topicType.add(topicType);
    }
    
    /**
     *
     * Minste lovlige constructor, alle topic m innholde minst en topicType
     * @param topicType  Forteller av hvilken typet emnet er av.<br><br>
     */
    public Topic(String topicType) {
        this.topicType = new ArrayList();
        this.topicType.add(topicType);
    }
    
    /**Metoden setter et topicDuid attributt.
     * @param topicDuid  Attributt som identifiserer topic'et unikt.<br><br>
     */
    public void setTopicDuid(URL topicDuid){
        this.topicDuid = topicDuid;
    }
    
    /**Metoden legger til "ekstra" comment element. Kan gjentaes s ofte som nskelig.
     * @param comment  Legger til tilleggs informasjon i naturlig sprk.<br><br>
     */
    public void addComment(String comment){
        if (this.comment == null) this.comment = new ArrayList();
        this.comment.add(comment);
        
    }
    
    /**Metoden legger til "ekstra" topicType element. Kan gjentaes s ofte som nskelig.
     * @param topicType Forteller av hvilken typet emnet er av.<br><br>
     */
    public void addTopicType(String topicType){
        this.topicType.add(topicType);
    }
    
    /**Metoden legger til "ekstra" description element som har "description" og "descriptionXmlLang" som parameter. Kan gjentaes s ofte som 
     *nskelig.
     * @param description           Beskriver emnet.<br><br>
     * @param descriptionXmlLang    Angir sprket beskrivelsen er gitt i.<br><br>
     */
    public void addDescriptionMap(String description, String descriptionXmlLang){
        if (this.descriptionMap == null) this.descriptionMap = new ArrayList();
        this.descriptionMap.add(new DescriptionMap(description, descriptionXmlLang));
    }
    
    /**Metoden legger til "ekstra" description element som kun har "description" som parameter. Kan gjentaes s ofte som nskelig.
     * @param description Beskriver emnet.<br><br>
     */
    public void addDescriptionMap(String description){
        if (this.descriptionMap == null) this.descriptionMap = new ArrayList();
        this.descriptionMap.add(new DescriptionMap(description));
    }
    
    /**Metoden legger til "ekstra" property element. Kan gjentaes s ofte som nskelig.
     * @param property  Angir verdien for spesifikke egenskaper ved emnet, som ikke er dekket av de vrige. <br><br>
     */
    public void addProperty(String property){
        if (this.property == null) this.property = new ArrayList();
        this.property.add(property);
    }
    
    /**
     * Metoden returnerer alle topic variablene, i xml-format.
     *
     */
    public String getTopic(){
        String t ="<Topic ";
        if (topicDuid != null) {t = t + "Duid=\"" + topicDuid + "\"";}
        t = t + ">";
        if (comment != null) {
            Iterator i = comment.iterator();
            while (i.hasNext()){
                t = t + "<Comment>" + ((String)i.next()).toString() + "</Comment>";
            }
        }
        if (catalog != null) {t = t+ "<Catalog>" + catalog.getCatalog() + "</Catalog>";}
        if (topicType != null){
            Iterator i = topicType.iterator();
            while (i.hasNext()) t = t + "<TopicType FormalName=\"" + i.next().toString() + "\"/>";
        }
        if (descriptionMap != null) {
            Iterator i = descriptionMap.iterator();
            while (i.hasNext()) t = t + i.next().toString();
        }
        if ( property != null) {
            Iterator i = property.iterator();
            while (i.hasNext()) t = t + "<Property FormalName=\"" + ((String)i.next()).toString() + "\" />";
        }
        t = t+ "</Topic>";
        return t;
    }
    
    
    ////////////////////////////////////////////////
    ////////Subclass DescriptionMap///////////////////
    //////////////////////////////////////////////
    /**
     *Klassen hndterer kombinereringen av "description" og "descriptionXmlLang".
     */
    private class DescriptionMap{
        private String description          = null; //0-N
        private String descriptionXmlLang   = null; //att
        
        public DescriptionMap(String description, String descriptionXmlLang){
            this.description        = description;
            this.descriptionXmlLang = descriptionXmlLang;
        }
        
        public DescriptionMap(String description){
            this.description        = description;
        }
        
        public String toString(){
            if (descriptionXmlLang != null)
                return new String("<Description xml:lang=\"" + descriptionXmlLang + "\">" + description + "</Description>");
            return new String("<Description>" + description + "</Description>");
        }
    }
}
