package no.ingeberg.NewsML;
import java.util.*;

/**
 * TopicSet.java<br>
 * Klassen hndterer innkomne data under "TopicSet" elementet, som beskriver emnet til deler eller hele nyhetsposter.
 * Klassen innholder metoder for  hente ut disse formatert som xml,
 * etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class TopicSet {
    private String topicSetFormalName   = null;
    private int commentNr               = 0;//comment er en string som kan ha 0-N forekomseter
    private ArrayList comment           = null;
    private Catalog catalog             = null;
    private int topicSetRefNr           = 0;// topicsetRef er en string som kan ha 0-N forekomseter
    private ArrayList topicSetRef       = null;
    private int topicNr                 = 0;//topic er et topic element som kan ha 0-N forekomster
    private ArrayList topic             = null;
    
    /**
     *Lager en ny instanse av instance of TopicSet, med alle tillatte
     *elemeneter
     * @param topicSetFormalName    Angir navn til "topicSet'et".<br><br>
     * @param comment               Frivillig kommentarfelt.<br><br>
     * @param catalog               Identifiserer standard vokabular for Topic'et. <br>
     *                              Den kan og indikerer hvor i NewsML dokumentet Topicet er brukt. <br>
     *                              Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param topicSetRef           Her kan en angi en peker til et TopicSet som skal fylle ut det aktuelle TopicSet'et.<br><br>
     * @param topic                 Objekt som angir selve "topic'et".
     */
    public TopicSet(String topicSetFormalName, String comment, Catalog catalog,
    String topicSetRef, Topic topic) {
        this.topicSetFormalName = topicSetFormalName;
        this.comment            = new ArrayList();
        this.catalog            = catalog;
        this.topicSetRef        = new ArrayList();
        this.topic              = new ArrayList();
        this.comment.add(comment);
        this.topicSetRef.add(topicSetRef);
        this.topic.add(topic);
    }
    /**
     * //konstruktor som lager et TopicSet objekt, som innholder catalog og Topic elementer
     * @param topicSetFormalName    Angir navn til "topicSet'et". <br><br>
     * @param catalog               Identifiserer standard vokabular for Topic'et. <br>
     *                              Den kan og indikerer hvor i NewsML dokumentet Topicet er brukt. <br>
     *                              Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param topic                 Objekt som angir selve "topic'et".<br>
     */
    public TopicSet(String topicSetFormalName, Catalog catalog, Topic topic) {
        this.topicSetFormalName = topicSetFormalName;
        this.catalog = catalog;
        this.topic = new ArrayList();
        this.topic.add(topic);
    }
     
    /**
     * //konstruktor som lager et TopicSet objekt, som innholder kun et Topic element
     * @param topic                 Objekt som angir selve "topic'et".<br>
     */
   public TopicSet(Topic topic) {
        this.topic = new ArrayList();
        this.topic.add(topic);
    }
  
    /**
     * Konstruktopren oppretter et minimus objekt. Elemetet TopicSet kan vre tomt,
     * men det m ha en FormalName attributt
     * @param topicSetFormalName    Angir navn til "topicSet'et".
     */
    public TopicSet(String topicSetFormalName) {
        this.topicSetFormalName = topicSetFormalName;
    }
    
    /**Metoden legger til "ekstra" comment element. Kan gjentaes s ofte som nskelig.
     * @param comment  Frivillig kommentarfelt.
     */
    public void addComment(String comment){
        if (this.comment == null) this.comment = new ArrayList();
        this.comment.add(comment);
    }
    
    /**Metoden legger til "ekstra" topicSetRef element. Kan gjentaes s ofte som nskelig.
     * @param topicSetRef  Her kan en angi en peker til et TopicSet som skal fylle ut det aktuelle TopicSet'et.
     */
    public void addTopicSetRef(String topicSetRef){
        if (this.topicSetRef == null) this.topicSetRef = new ArrayList();
        this.topicSetRef.add(topicSetRef);
    }
    
    /**Metoden legger til et "ekstra" topic element, Kan gjentaes s ofte som nskelig.
     * @param topic Objekt som angir selve "topic'et".
     */
    public void addTopic(Topic topic){
        if (this.topic == null) this.topic = new ArrayList();
        this.topic.add(topic);
    }
    
    /**
     * // Metoden returnerer alle topic variablene, i xml-format.
     * @return 
     */
    public String getTopicSet(){
        //
        String t = "<TopicSet ";
        if (topicSetFormalName !=null){t = t+ "FormalName=\"" + topicSetFormalName + "\"";}
        t = t + ">";
        if (comment !=null){
            Iterator i = comment.iterator();
            while (i.hasNext()){
                t = t + "<Comment>" + ((String)i.next()).toString() + "</Comment>";
            }
        }
        if (catalog !=null){t = t +  catalog.getCatalog();}
        if (topicSetRef !=null){
            Iterator i = comment.iterator();
            while (i.hasNext()){
                t = t + "<TopicSetRef TopicSet=\"" + ((String)i.next()).toString() + " \"/>";
            }
        }
        if (topic !=null){
            Iterator i = topic.iterator();
            while (i.hasNext()){
                t = t + ((no.ingeberg.NewsML.Topic)i.next()).getTopic();
            }
        }
        t = t + "</TopicSet>";
        return t;
    }
}
