/*
 * Kalender.java
 *
 */

package no.ingeberg.newsMLGrensesnitt.tools;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import no.ingeberg.newsMLGrensesnitt.*;

public class Kalender extends JFrame implements ActionListener {
    AdminBaseGUI parent = null;
    JTextField navn = null;
    
    /** Labelen med innevrende r og mned */
    private JLabel mthLabel;
    /** Tabellen med kalenderen i */
    private JTable calTable;
    /** Kalendermned */
    private int month = 0;
    /** r */
    private int year = 1970;
    /** Frste ukedag i denne mneden */
    private int weekday = 3;
    /** Ukedager */
    private static String[] weekdays = { "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lrdag", "Sndag" };
    /** Mneder */
    private static String[] months = { "Januar", "Februar", "Mars", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November",
                                        "Desember" };
    
    /**
     * Konstruktor.
     */
    public Kalender(AdminBaseGUI parent, JTextField navn) {
        super("Kalender");
        gotoCurrent();
        init();
        showMonth();
        this.parent = parent;
        this.navn = navn;
    }
    
    /**
     * Hopper frem en mned.
     */
    private void monthForward() {
        weekday = (weekday + daysInMonth(month, year)) % 7;
        month += 1;
        if (month > 11) {
            month = 0;
            year += 1;
        }
    }
    
    /**
     * Hopper tilbake en mned.
     */
    private void monthBack() {
        month -= 1;
        if (month < 0) {
            month = 11;
            year -= 1;
        }
        weekday = (weekday - daysInMonth(month, year)) % 7;
        if (weekday < 0) { weekday += 7; }
    }
    
    /**
     * Hopper fram et r.
     */
    private void yearForward() {
        for (int i = 0; i < 12; i++) {
            monthForward();
        }
    }
    
    /**
     * Hopper tilbake et r.
     */
    private void yearBack() {
        for (int i = 0; i < 12; i++) {
            monthBack();
        }
    }
    
    /**
     * Gr til innevrende mned... et utmerket sted  starte.
     */
    private void gotoCurrent() {
        long days = System.currentTimeMillis();
        days /= 86400000;
        while (days > daysInYear(year)) {
            days -= daysInYear(year);
            yearForward();
        }
        while (days > daysInMonth(month, year)) {
            days -= daysInMonth(month, year);
            monthForward();
        }
    }
    
    /**
     * Returnerer antall dager i et gitt r.
     *
     * @param yr rstallet det er sprsml om
     */
    private static int daysInYear(int yr) {
        if (yr % 400 == 0) {
            return 366;
        } else if (yr % 100 == 0) {
            return 365;
        } else if (yr % 4 == 0) {
            return 366;
        } else {
            return 365;
        }
    }
    
    /**
     * Returnerer antall dager i en gitt mned.
     *
     * @param mth mneden det er sprsml om
     * @param yr rstallet det er sprsml om
     */
    private static int daysInMonth(int mth, int yr) throws ArrayIndexOutOfBoundsException {
        switch (mth) {
            case 0:
            case 2:
            case 4:
            case 6:
            case 7:
            case 9:
            case 11:
                return 31;
            case 3:
            case 5:
            case 8:
            case 10:
                return 30;
            case 1:
                if (yr % 400 == 0) {
                    return 29;
                } else if (yr % 100 == 0) {
                    return 28;
                } else if (yr % 4 == 0) {
                    return 29;
                } else {
                    return 28;
                }
            default:
                throw new ArrayIndexOutOfBoundsException("There is no month number " + mth + " in the year!");
        }
    }
    
    /**
     * Viser n-mneden i tabellen.
     */
    private void showMonth() {
        Object[][] input = new Object[7][7];
        input[0] = weekdays;
        int j = 1;
        int k = weekday;
        for (int i = 1; i <= daysInMonth(month, year); i++) {
            input[j][k] = Integer.toString(i);
            k++;
            if (k > 6) {
                k = 0;
                j++;
            }
        }
        calTable.setModel(new DefaultTableModel(input, weekdays));
        mthLabel.setText(months[month] + " " + year);
    }
    
    /**
     * Vindusinitiering. Starter opp og legger opp vinduet.
     */
    private void init() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        java.awt.GridBagConstraints gridBagConstraints;
        
        JPanel panel1 = new JPanel();
        JPanel panel2 = new JPanel();
        JPanel panel3 = new JPanel();
        JPanel panel4 = new JPanel();
        mthLabel = new JLabel();
        JLabel label1 = new JLabel();
        calTable = new JTable();
        JButton bt1 = new JButton("<<");
        JButton bt2 = new JButton("<");
        JButton bt3 = new JButton(">");
        JButton bt4 = new JButton(">>");
        
        panel1.setLayout(new java.awt.BorderLayout());
        
        mthLabel.setText("Mned :");
        panel4.add(mthLabel);
        
        panel1.add(panel4, BorderLayout.NORTH);
        calTable.setModel(
        new DefaultTableModel(
        new Object [][] {
            weekdays,
            {null, null, null, null, null, null, null},
            {null, null, null, null, null, null, null},
            {null, null, null, null, null, null, null},
            {null, null, null, null, null, null, null},
            {null, null, null, null, null, null, null},
            {null, null, null, null, null, null, null}
        },
        weekdays
        )
        );
        panel1.add(calTable, java.awt.BorderLayout.CENTER);
        getContentPane().add(panel1, BorderLayout.CENTER);
        
        label1.setFont(new java.awt.Font("Dialog", 1, 18));
        label1.setText("Kalender");
        panel3.add(label1);
        getContentPane().add(panel3, BorderLayout.NORTH);
        
        bt1.addActionListener(this);
        panel2.add(bt1);
        bt2.addActionListener(this);
        panel2.add(bt2);
        bt3.addActionListener(this);
        panel2.add(bt3);
        bt4.addActionListener(this);
        panel2.add(bt4);
        
        calTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                getdate(evt);
            }
        });
        
        getContentPane().add(panel2, BorderLayout.SOUTH);
        
        pack();
        setVisible(true);
        setResizable(false);
    }
    
    public void getdate(java.awt.event.MouseEvent evt) {
        String sDag =(String)calTable.getModel().getValueAt(calTable.getSelectedRow(),calTable.getSelectedColumn());
        String dato;
        int maane = month;
        maane++;
        if (maane < 10) dato = "0"+new Integer(maane).toString();
        else dato = new Integer(maane).toString();
        
        int dag = new Integer(sDag).intValue();
        if (dag < 10) sDag = "0" +new Integer(dag).toString();
        else sDag = new Integer(dag).toString();
        
        dato = new Integer(year).toString()+dato+sDag;
        parent.setDato(dato, navn);
    }
    /**
     * Noen trykket p en knapp, hva skal skje n?
     *
     * @param ActionEvent e eventen som ble utlst
     */
    public void actionPerformed(ActionEvent e) {
        String cmd = ((JButton)e.getSource()).getText();
        if (cmd.equals("<<")) {
            yearBack();
        }
        if (cmd.equals("<")) {
            monthBack();
        }
        if (cmd.equals(">")) {
            monthForward();
        }
        if (cmd.equals(">>")) {
            yearForward();
        }
        showMonth();
    }
}